@@ -64,7 +64,8 @@ public class CHNativeExpressionEvaluator extends ExpressionEvaluatorJniWrapper {
       byte[] wsPlan,
       byte[][] splitInfo,
       List<ColumnarNativeIterator> iterList,
-      boolean materializeInput) {
+      boolean materializeInput,
+      int partitionIndex) {
     CHThreadGroup.registerNewThreadGroup();
     long handle =
         nativeCreateKernelWithIterator(
@@ -72,13 +73,17 @@ public class CHNativeExpressionEvaluator extends ExpressionEvaluatorJniWrapper {
             splitInfo,
             iterList.toArray(new ColumnarNativeIterator[0]),
             ConfigUtil.serialize(getNativeBackendConf()),
-            materializeInput);
+            materializeInput,
+            partitionIndex);
     return createBatchIterator(handle);
   }
 
   // Only for UT.
   public static BatchIterator createKernelWithBatchIterator(
-      byte[] wsPlan, byte[][] splitInfo, List<ColumnarNativeIterator> iterList) {
+      byte[] wsPlan,
+      byte[][] splitInfo,
+      List<ColumnarNativeIterator> iterList,
+      int partitionIndex) {
     CHThreadGroup.registerNewThreadGroup();
     long handle =
         nativeCreateKernelWithIterator(
@@ -86,7 +91,8 @@ public class CHNativeExpressionEvaluator extends ExpressionEvaluatorJniWrapper {
             splitInfo,
             iterList.toArray(new ColumnarNativeIterator[0]),
             ConfigUtil.serialize(getNativeBackendConf()),
-            false);
+            false,
+            partitionIndex);
     return createBatchIterator(handle);
   }
 
