@@ -93,9 +93,8 @@ object RunTPCHTest {
       .config("spark.databricks.delta.snapshotPartitions", 1)
       .config("spark.databricks.delta.properties.defaults.checkpointInterval", 5)
       .config("spark.databricks.delta.stalenessLimit", 3600 * 1000)
-      .config("spark.gluten.sql.columnar.columnarToRow", columnarColumnToRow)
       .config(ClickHouseConfig.CLICKHOUSE_WORKER_ID, "1")
-      .config(GlutenConfig.GLUTEN_LIB_PATH, libPath)
+      .config(GlutenConfig.GLUTEN_LIB_PATH.key, libPath)
       .config("spark.gluten.sql.columnar.iterator", "true")
       .config("spark.gluten.sql.columnar.hashagg.enablefinal", "true")
       .config("spark.gluten.sql.enable.native.validation", "false")
