@@ -17,7 +17,6 @@
 package org.apache.gluten.execution
 
 import org.apache.gluten.backendsapi.clickhouse.RuntimeConfig
-import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.utils.UTSystemParameters
 
 import org.apache.spark.{SPARK_VERSION_SHORT, SparkConf}
@@ -42,15 +41,17 @@ class GlutenClickHouseWholeStageTransformerSuite extends WholeStageTransformerSu
   }
   val SPARK_DIR_NAME: String = sparkVersion.replace(".", "-")
 
-  val S3_METADATA_PATH = s"/tmp/metadata/s3/$SPARK_DIR_NAME"
-  val S3_CACHE_PATH = s"/tmp/s3_cache/$SPARK_DIR_NAME/"
+  private val TMP_PREFIX = s"/tmp/gluten/$SPARK_DIR_NAME"
+
+  val S3_METADATA_PATH = s"$TMP_PREFIX/s3/metadata"
+  val S3_CACHE_PATH = s"$TMP_PREFIX/s3/cache"
   val S3_ENDPOINT = "s3://127.0.0.1:9000/"
   val MINIO_ENDPOINT: String = S3_ENDPOINT.replace("s3", "http")
   val BUCKET_NAME: String = SPARK_DIR_NAME
   val WHOLE_PATH: String = MINIO_ENDPOINT + BUCKET_NAME + "/"
 
-  val HDFS_METADATA_PATH = s"/tmp/metadata/hdfs/$SPARK_DIR_NAME"
-  val HDFS_CACHE_PATH = s"/tmp/hdfs_cache/$SPARK_DIR_NAME/"
+  val HDFS_METADATA_PATH = s"$TMP_PREFIX/hdfs/metadata"
+  val HDFS_CACHE_PATH = s"$TMP_PREFIX/hdfs/cache"
   val HDFS_URL_ENDPOINT = "hdfs://127.0.0.1:8020"
   val HDFS_URL = s"$HDFS_URL_ENDPOINT/$SPARK_DIR_NAME"
 
@@ -59,6 +60,8 @@ class GlutenClickHouseWholeStageTransformerSuite extends WholeStageTransformerSu
 
   val CH_DEFAULT_STORAGE_DIR = "/data"
 
+  val LOCAL_CACHE_PATH = s"$TMP_PREFIX/local/cache"
+
   protected def spark32: Boolean = sparkVersion.equals("3.2")
   protected def spark33: Boolean = sparkVersion.equals("3.3")
   protected def spark35: Boolean = sparkVersion.equals("3.5")
@@ -76,10 +79,9 @@ class GlutenClickHouseWholeStageTransformerSuite extends WholeStageTransformerSu
   }
 
   override protected def sparkConf: SparkConf = {
-    import org.apache.gluten.backendsapi.clickhouse.CHConf._
+    import org.apache.gluten.backendsapi.clickhouse.CHConfig._
 
     val conf = super.sparkConf
-      .set(GlutenConfig.GLUTEN_LIB_PATH, UTSystemParameters.clickHouseLibPath)
       .set("spark.gluten.sql.enable.native.validation", "false")
       .set("spark.sql.warehouse.dir", warehouse)
       .setCHConfig("user_defined_path", "/tmp/user_defined")
