@@ -17,6 +17,7 @@
 package org.apache.gluten.backendsapi.velox
 
 import org.apache.gluten.backendsapi.{BackendsApiManager, IteratorApi}
+import org.apache.gluten.backendsapi.velox.VeloxIteratorApi.unescapePathName
 import org.apache.gluten.config.GlutenNumaBindingInfo
 import org.apache.gluten.execution._
 import org.apache.gluten.iterator.Iterators
@@ -25,7 +26,6 @@ import org.apache.gluten.sql.shims.SparkShimLoader
 import org.apache.gluten.substrait.plan.PlanNode
 import org.apache.gluten.substrait.rel.{LocalFilesBuilder, LocalFilesNode, SplitInfo}
 import org.apache.gluten.substrait.rel.LocalFilesNode.ReadFileFormat
-import org.apache.gluten.utils._
 import org.apache.gluten.vectorized._
 
 import org.apache.spark.{SparkConf, TaskContext}
@@ -65,7 +65,8 @@ class VeloxIteratorApi extends IteratorApi with Logging {
           fileSizes,
           modificationTimes,
           partitionColumns,
-          metadataColumns) =
+          metadataColumns,
+          otherMetadataColumns) =
           constructSplitInfo(partitionSchema, f.files, metadataColumnNames)
         val preferredLocations =
           SoftAffinity.getFilePartitionLocations(f)
@@ -80,18 +81,62 @@ class VeloxIteratorApi extends IteratorApi with Logging {
           metadataColumns,
           fileFormat,
           preferredLocations.toList.asJava,
-          mapAsJavaMap(properties)
+          mapAsJavaMap(properties),
+          otherMetadataColumns
         )
       case _ =>
         throw new UnsupportedOperationException(s"Unsupported input partition.")
     }
   }
 
+  override def genSplitInfoForPartitions(
+      partitionIndex: Int,
+      partitions: Seq[InputPartition],
+      partitionSchema: StructType,
+      fileFormat: ReadFileFormat,
+      metadataColumnNames: Seq[String],
+      properties: Map[String, String]): SplitInfo = {
+    val partitionFiles = partitions.flatMap {
+      p =>
+        if (!p.isInstanceOf[FilePartition]) {
+          throw new UnsupportedOperationException(
+            s"Unsupported input partition ${p.getClass.getName}.")
+        }
+        p.asInstanceOf[FilePartition].files
+    }.toArray
+    val locations =
+      partitions.flatMap(p => SoftAffinity.getFilePartitionLocations(p.asInstanceOf[FilePartition]))
+    val (
+      paths,
+      starts,
+      lengths,
+      fileSizes,
+      modificationTimes,
+      partitionColumns,
+      metadataColumns,
+      otherMetadataColumns) =
+      constructSplitInfo(partitionSchema, partitionFiles, metadataColumnNames)
+    LocalFilesBuilder.makeLocalFiles(
+      partitionIndex,
+      paths,
+      starts,
+      lengths,
+      fileSizes,
+      modificationTimes,
+      partitionColumns,
+      metadataColumns,
+      fileFormat,
+      locations.toList.asJava,
+      mapAsJavaMap(properties),
+      otherMetadataColumns
+    )
+  }
+
   /** Generate native row partition. */
   override def genPartitions(
       wsCtx: WholeStageTransformContext,
       splitInfos: Seq[Seq[SplitInfo]],
-      scans: Seq[BasicScanExecTransformer]): Seq[BaseGlutenPartition] = {
+      leaves: Seq[LeafTransformSupport]): Seq[BaseGlutenPartition] = {
     // Only serialize plan once, save lots time when plan is complex.
     val planByteArray = wsCtx.root.toProtobuf.toByteArray
 
@@ -117,13 +162,10 @@ class VeloxIteratorApi extends IteratorApi with Logging {
     val modificationTimes = new JArrayList[JLong]()
     val partitionColumns = new JArrayList[JMap[String, String]]
     val metadataColumns = new JArrayList[JMap[String, String]]
+    val otherMetadataColumns = new JArrayList[JMap[String, Object]]
     files.foreach {
       file =>
-        // The "file.filePath" in PartitionedFile is not the original encoded path, so the decoded
-        // path is incorrect in some cases and here fix the case of ' ' by using GlutenURLDecoder
-        paths.add(
-          GlutenURLDecoder
-            .decode(file.filePath.toString, StandardCharsets.UTF_8.name()))
+        paths.add(unescapePathName(file.filePath.toString))
         starts.add(JLong.valueOf(file.start))
         lengths.add(JLong.valueOf(file.length))
         val (fileSize, modificationTime) =
@@ -160,8 +202,18 @@ class VeloxIteratorApi extends IteratorApi with Logging {
           partitionColumn.put(schema.names(i), partitionColumnValue)
         }
         partitionColumns.add(partitionColumn)
+        otherMetadataColumns.add(
+          SparkShimLoader.getSparkShims.getOtherConstantMetadataColumnValues(file))
     }
-    (paths, starts, lengths, fileSizes, modificationTimes, partitionColumns, metadataColumns)
+    (
+      paths,
+      starts,
+      lengths,
+      fileSizes,
+      modificationTimes,
+      partitionColumns,
+      metadataColumns,
+      otherMetadataColumns)
   }
 
   override def injectWriteFilesTempPath(path: String, fileName: String): Unit = {
@@ -269,3 +321,51 @@ class VeloxIteratorApi extends IteratorApi with Logging {
   }
   // scalastyle:on argcount
 }
+
+object VeloxIteratorApi {
+  // lookup table to translate '0' -> 0 ... 'F'/'f' -> 15
+  private val unhexDigits = {
+    val array = Array.fill[Byte](128)(-1)
+    (0 to 9).foreach(i => array('0' + i) = i.toByte)
+    (0 to 5).foreach(i => array('A' + i) = (i + 10).toByte)
+    (0 to 5).foreach(i => array('a' + i) = (i + 10).toByte)
+    array
+  }
+
+  def unescapePathName(path: String): String = {
+    if (path == null || path.isEmpty) {
+      return path
+    }
+    var plaintextEndIdx = path.indexOf('%')
+    val length = path.length
+    if (plaintextEndIdx == -1 || plaintextEndIdx + 2 >= length) {
+      // fast path, no %xx encoding found then return the string identity
+      path
+    } else {
+      val sb = new java.lang.StringBuilder(length)
+      var plaintextStartIdx = 0
+      while (plaintextEndIdx != -1 && plaintextEndIdx + 2 < length) {
+        if (plaintextEndIdx > plaintextStartIdx) sb.append(path, plaintextStartIdx, plaintextEndIdx)
+        val high = path.charAt(plaintextEndIdx + 1)
+        if ((high >>> 8) == 0 && unhexDigits(high) != -1) {
+          val low = path.charAt(plaintextEndIdx + 2)
+          if ((low >>> 8) == 0 && unhexDigits(low) != -1) {
+            sb.append((unhexDigits(high) << 4 | unhexDigits(low)).asInstanceOf[Char])
+            plaintextStartIdx = plaintextEndIdx + 3
+          } else {
+            sb.append('%')
+            plaintextStartIdx = plaintextEndIdx + 1
+          }
+        } else {
+          sb.append('%')
+          plaintextStartIdx = plaintextEndIdx + 1
+        }
+        plaintextEndIdx = path.indexOf('%', plaintextStartIdx)
+      }
+      if (plaintextStartIdx < length) {
+        sb.append(path, plaintextStartIdx, length)
+      }
+      sb.toString
+    }
+  }
+}
