@@ -17,9 +17,10 @@
 package org.apache.gluten.execution
 
 import org.apache.gluten.config.GlutenConfig
+import org.apache.gluten.config.VeloxConfig
 
 import org.apache.spark.SparkConf
-import org.apache.spark.sql.{DataFrame, Row, TestUtils}
+import org.apache.spark.sql.{DataFrame, GlutenTestUtils, Row}
 import org.apache.spark.sql.execution.{ColumnarShuffleExchangeExec, FormattedMode}
 
 import org.apache.commons.io.FileUtils
@@ -264,7 +265,7 @@ class VeloxTPCHDistinctSpillSuite extends VeloxTPCHTableSupport {
 
   test("distinct spill") {
     val df = spark.sql("select count(distinct *) from lineitem limit 1")
-    TestUtils.compareAnswers(df.collect(), Seq(Row(60175)))
+    GlutenTestUtils.compareAnswers(df.collect(), Seq(Row(60175)))
   }
 }
 
@@ -286,7 +287,7 @@ class VeloxTPCHMiscSuite extends VeloxTPCHTableSupport {
     val result = df.collect()
     df.explain(true)
     val expectedResult = Seq(Row(0), Row(1), Row(2), Row(3), Row(4))
-    TestUtils.compareAnswers(result, expectedResult)
+    GlutenTestUtils.compareAnswers(result, expectedResult)
   }
 }
 
@@ -344,7 +345,7 @@ class VeloxTPCHV1BhjOffheapSuite extends VeloxTPCHSuite {
     super.sparkConf
       .set("spark.sql.sources.useV1SourceList", "parquet")
       .set("spark.sql.autoBroadcastJoinThreshold", "30M")
-      .set(GlutenConfig.VELOX_BROADCAST_BUILD_RELATION_USE_OFFHEAP.key, "true")
+      .set(VeloxConfig.VELOX_BROADCAST_BUILD_RELATION_USE_OFFHEAP.key, "true")
   }
 }
 
