@@ -25,6 +25,7 @@
 #include <Parser/TypeParser.h>
 #include <Common/BlockTypeUtils.h>
 #include <Common/CHUtil.h>
+#include <Common/logger_useful.h>
 #include "ExpressionParser.h"
 
 namespace DB
@@ -41,7 +42,8 @@ namespace local_engine
 {
 using namespace DB;
 
-FunctionParser::FunctionParser(ParserContextPtr ctx) : parser_context(ctx)
+FunctionParser::FunctionParser(ParserContextPtr ctx)
+    : parser_context(ctx)
 {
     expression_parser = std::make_unique<ExpressionParser>(parser_context);
 }
@@ -121,7 +123,7 @@ const ActionsDAG::Node * FunctionParser::convertNodeTypeIfNeeded(
     {
         if (!TypeParser::isTypeMatched(output_type, func_node->result_type))
         {
-            auto result_type = TypeParser::parseType(substrait_func.output_type());
+            auto result_type = TypeParser::parseType(output_type);
             if (DB::isDecimalOrNullableDecimal(result_type))
             {
                 return ActionsDAGUtil::convertNodeType(
@@ -135,13 +137,10 @@ const ActionsDAG::Node * FunctionParser::convertNodeTypeIfNeeded(
             }
             else
             {
-                return ActionsDAGUtil::convertNodeType(
-                    actions_dag,
-                    func_node,
-                    // as stated in isTypeMatched， currently we don't change nullability of the result type
-                    func_node->result_type->isNullable() ? local_engine::wrapNullableType(true, TypeParser::parseType(output_type))
-                                                         : DB::removeNullable(TypeParser::parseType(output_type)),
-                    func_node->result_name);
+                // as stated in isTypeMatched， currently we don't change nullability of the result type
+                auto target_type = func_node->result_type->isNullable() ? local_engine::wrapNullableType(true, result_type)
+                                                                        : local_engine::removeNullable(result_type);
+                return ActionsDAGUtil::convertNodeType(actions_dag, func_node, target_type, func_node->result_name);
             }
         }
         else
