@@ -20,6 +20,7 @@
 #include <Core/Settings.h>
 #include <Functions/FunctionFactory.h>
 #include <Interpreters/CollectJoinOnKeysVisitor.h>
+#include <Interpreters/ExpressionActions.h>
 #include <Interpreters/FullSortingMergeJoin.h>
 #include <Interpreters/GraceHashJoin.h>
 #include <Interpreters/HashJoin/HashJoin.h>
@@ -30,6 +31,7 @@
 #include <Parser/AdvancedParametersParseUtil.h>
 #include <Parser/ExpressionParser.h>
 #include <Parsers/ASTIdentifier.h>
+#include <Parser/SubstraitParserUtils.h>
 #include <Processors/QueryPlan/ExpressionStep.h>
 #include <Processors/QueryPlan/FilterStep.h>
 #include <Processors/QueryPlan/JoinStep.h>
@@ -45,6 +47,8 @@ namespace Setting
 extern const SettingsJoinAlgorithm join_algorithm;
 extern const SettingsUInt64 max_block_size;
 extern const SettingsUInt64 min_joined_block_size_bytes;
+extern const SettingsNonZeroUInt64 grace_hash_join_initial_buckets;
+extern const SettingsNonZeroUInt64 grace_hash_join_max_buckets;
 }
 namespace ErrorCodes
 {
@@ -57,14 +61,17 @@ using namespace DB;
 
 namespace local_engine
 {
-std::shared_ptr<DB::TableJoin> createDefaultTableJoin(substrait::JoinRel_JoinType join_type, bool is_existence_join, ContextPtr & context)
+std::shared_ptr<DB::TableJoin> createDefaultTableJoin(substrait::JoinRel_JoinType join_type, const JoinOptimizationInfo & join_opt_info, ContextPtr & context)
 {
     auto table_join
         = std::make_shared<TableJoin>(context->getSettingsRef(), context->getGlobalTemporaryVolume(), context->getTempDataOnDisk());
 
-    std::pair<DB::JoinKind, DB::JoinStrictness> kind_and_strictness = JoinUtil::getJoinKindAndStrictness(join_type, is_existence_join);
+    std::pair<DB::JoinKind, DB::JoinStrictness> kind_and_strictness = JoinUtil::getJoinKindAndStrictness(join_type, join_opt_info.is_existence_join);
     table_join->setKind(kind_and_strictness.first);
-    table_join->setStrictness(kind_and_strictness.second);
+    if (!join_opt_info.is_any_join)
+        table_join->setStrictness(kind_and_strictness.second);
+    else
+        table_join->setStrictness(DB::JoinStrictness::Any);
     return table_join;
 }
 
@@ -111,10 +118,9 @@ std::unordered_set<DB::JoinTableSide> JoinRelParser::extractTableSidesFromExpres
             table_sides.insert(table_sides_from_arg.begin(), table_sides_from_arg.end());
         }
     }
-    else if (expr.has_selection() && expr.selection().has_direct_reference() && expr.selection().direct_reference().has_struct_field())
+    else if (auto field = SubstraitParserUtils::getStructFieldIndex(expr))
     {
-        auto pos = expr.selection().direct_reference().struct_field().field();
-        if (pos < left_header.columns())
+        if (*field < left_header.columns())
             table_sides.insert(DB::JoinTableSide::Left);
         else
             table_sides.insert(DB::JoinTableSide::Right);
@@ -203,7 +209,7 @@ DB::QueryPlanPtr JoinRelParser::parseJoin(const substrait::JoinRel & join, DB::Q
     if (storage_join)
         renamePlanColumns(*left, *right, *storage_join);
 
-    auto table_join = createDefaultTableJoin(join.type(), join_opt_info.is_existence_join, context);
+    auto table_join = createDefaultTableJoin(join.type(), join_opt_info, context);
     DB::Block right_header_before_convert_step = right->getCurrentHeader();
     addConvertStep(*table_join, *left, *right);
 
@@ -269,15 +275,10 @@ DB::QueryPlanPtr JoinRelParser::parseJoin(const substrait::JoinRel & join, DB::Q
                 auto input_header = left->getCurrentHeader();
                 DB::ActionsDAG filter_is_not_null_dag{input_header.getColumnsWithTypeAndName()};
                 // when is_null_aware_anti_join is true, there is only one join key
-                const auto * key_field = filter_is_not_null_dag.getInputs()[join.expression()
-                                                                                .scalar_function()
-                                                                                .arguments()
-                                                                                .at(0)
-                                                                                .value()
-                                                                                .selection()
-                                                                                .direct_reference()
-                                                                                .struct_field()
-                                                                                .field()];
+                auto field_index = SubstraitParserUtils::getStructFieldIndex(join.expression().scalar_function().arguments(0).value());
+                if (!field_index)
+                    throw DB::Exception(DB::ErrorCodes::LOGICAL_ERROR, "The join key is not found in the expression.");
+                const auto * key_field = filter_is_not_null_dag.getInputs()[*field_index];
 
                 auto result_node = filter_is_not_null_dag.tryFindInOutputs(key_field->result_name);
                 // add a function isNotNull to filter the null key on the left side
@@ -477,12 +478,12 @@ void JoinRelParser::collectJoinKeys(
             size_t left_pos = 0, right_pos = 0;
             for (const auto & arg : current_expr->scalar_function().arguments())
             {
-                if (!arg.value().has_selection() || !arg.value().selection().has_direct_reference()
-                    || !arg.value().selection().direct_reference().has_struct_field())
+                auto field_index = SubstraitParserUtils::getStructFieldIndex(arg.value());
+                if (!field_index)
                 {
                     throw DB::Exception(DB::ErrorCodes::LOGICAL_ERROR, "A column reference is expected");
                 }
-                auto col_pos_ref = arg.value().selection().direct_reference().struct_field().field();
+                auto col_pos_ref = *field_index;
                 if (col_pos_ref < left_header.columns())
                 {
                     left_pos = col_pos_ref;
@@ -547,8 +548,7 @@ bool JoinRelParser::applyJoinFilter(
         std::vector<substrait::Expression> exprs;
         for (size_t i = 0; i < header.columns(); ++i)
         {
-            substrait::Expression expr;
-            expr.mutable_selection()->mutable_direct_reference()->mutable_struct_field()->set_field(i);
+            substrait::Expression expr = SubstraitParserUtils::buildStructFieldExpression(i);
             exprs.emplace_back(expr);
         }
         return exprs;
@@ -677,23 +677,17 @@ bool JoinRelParser::couldRewriteToMultiJoinOnClauses(
         dfs_visit_and_expr(and_exprs, args[1].value());
     };
 
-    auto get_field_ref = [](const substrait::Expression & e) -> std::optional<Int32>
-    {
-        if (e.has_selection() && e.selection().has_direct_reference() && e.selection().direct_reference().has_struct_field())
-            return std::optional<Int32>(e.selection().direct_reference().struct_field().field());
-        return {};
-    };
     auto visit_equal_expr = [&](const substrait::Expression & e) -> std::optional<std::pair<String, String>>
     {
         if (!check_function("equals", e))
             return {};
         const auto & args = e.scalar_function().arguments();
-        auto l_field_ref = get_field_ref(args[0].value());
-        auto r_field_ref = get_field_ref(args[1].value());
+        auto l_field_ref = SubstraitParserUtils::getStructFieldIndex(args[0].value());
+        auto r_field_ref = SubstraitParserUtils::getStructFieldIndex(args[1].value());
         if (!l_field_ref.has_value() || !r_field_ref.has_value())
             return {};
-        size_t l_pos = static_cast<size_t>(*l_field_ref);
-        size_t r_pos = static_cast<size_t>(*r_field_ref);
+        size_t l_pos = *l_field_ref;
+        size_t r_pos = *r_field_ref;
         size_t l_cols = left_header.columns();
         size_t total_cols = l_cols + right_header.columns();
 
@@ -781,7 +775,9 @@ DB::QueryPlanPtr JoinRelParser::buildSingleOnClauseHashJoin(
     if (join_algorithm.isSet(DB::JoinAlgorithm::GRACE_HASH))
     {
         hash_join = std::make_shared<GraceHashJoin>(
-            context, table_join, left_plan->getCurrentHeader(), right_plan->getCurrentHeader(), context->getTempDataOnDisk());
+            context->getSettingsRef()[Setting::grace_hash_join_initial_buckets],
+            context->getSettingsRef()[Setting::grace_hash_join_max_buckets],
+            table_join, left_plan->getCurrentHeader(), right_plan->getCurrentHeader(), context->getTempDataOnDisk());
     }
     else
     {
