@@ -15,17 +15,19 @@
  * limitations under the License.
  */
 #include <iostream>
+#include <DataTypes/DataTypeNullable.h>
+#include <DataTypes/DataTypesNumber.h>
 #include <Parser/FunctionExecutor.h>
 #include <Parser/FunctionParser.h>
 #include <gtest/gtest.h>
-#include <DataTypes/DataTypeNullable.h>
+#include <Common/QueryContext.h>
 
 using namespace DB;
 using namespace local_engine;
 
 TEST(MyAdd, Common)
 {
-    auto context = local_engine::SerializedPlanParser::global_context;
+    auto context = local_engine::QueryContext::globalContext();
     auto type = std::make_shared<DataTypeNullable>(std::make_shared<DataTypeInt64>());
     FunctionExecutor executor("my_add", {type, type}, type, context);
 
