@@ -16,6 +16,7 @@
  */
 #pragma once
 
+#include <Columns/ColumnNullable.h>
 #include <Core/Block.h>
 #include <Core/Field.h>
 #include <Interpreters/Context.h>
@@ -33,7 +34,7 @@
 #include <Common/ArenaUtils.h>
 #include <Common/BlockTypeUtils.h>
 #include <Common/CHUtil.h>
-#include <Common/FieldVisitorsAccurateComparison.h>
+#include <Common/FieldAccurateComparison.h>
 
 namespace local_engine
 {
@@ -47,10 +48,12 @@ public:
     NormalFileWriter(const OutputFormatFilePtr & file_, const DB::ContextPtr & context_);
     ~NormalFileWriter() override = default;
 
-    void write(DB::Block & block) override;
+    void write(const DB::Block & block) override;
     void close() override;
 
 private:
+    DB::Block castBlock(const DB::Block & block) const;
+
     OutputFormatFilePtr file;
     DB::ContextPtr context;
 
@@ -65,6 +68,8 @@ OutputFormatFilePtr createOutputFormatFile(
 struct DeltaStats
 {
     size_t row_count;
+    // TODO Support delta.dataSkippingStatsColumns, detail see https://docs.databricks.com/aws/en/delta/data-skipping
+    size_t n_stats_cols;
     std::vector<DB::Field> min;
     std::vector<DB::Field> max;
     std::vector<Int64> null_count;
@@ -88,23 +93,55 @@ struct DeltaStats
 
         assert(partition_index.size() == partition.size());
 
+        size_t num_stats_cols = numStatsCols(output.columns() - partition.size());
         auto appendBase = [&](const std::string & prefix)
         {
-            for (const auto & column : output.getColumnsWithTypeAndName())
+            for (size_t i = 0, n = 0; i < output.columns() && n < num_stats_cols; i++)
+            {
+                const auto & column = output.getByPosition(i);
                 if (!partition_index.contains(column.name))
+                {
                     statsHeaderBase.emplace_back(wrapNullableType(column.type), prefix + column.name);
+                    ++n;
+                }
+            }
         };
         appendBase("min_");
         appendBase("max_");
-        for (const auto & column : output.getColumnsWithTypeAndName())
+        for (size_t i = 0, n = 0; i < output.columns() && n < num_stats_cols; i++)
+        {
+            const auto & column = output.getByPosition(i);
             if (!partition_index.contains(column.name))
+            {
                 statsHeaderBase.emplace_back(BIGINT(), "null_count_" + column.name);
+                ++n;
+            }
+        }
 
-        return makeBlockHeader(statsHeaderBase);
+        return DB::Block{statsHeaderBase};
+    }
+
+    static size_t numStatsCols(size_t origin)
+    {
+        if (DB::CurrentThread::isInitialized())
+        {
+            const DB::ContextPtr query_context = DB::CurrentThread::get().getQueryContext();
+            if (query_context)
+            {
+                SparkSQLConfig config = SparkSQLConfig::loadFromContext(query_context);
+                return std::min(config.deltaDataSkippingNumIndexedCols, origin);
+            }
+        }
+        return origin;
     }
 
     explicit DeltaStats(size_t size, const std::set<size_t> & partition_index_ = {})
-        : row_count(0), min(size), max(size), null_count(size, 0), partition_index(partition_index_)
+        : row_count(0)
+        , n_stats_cols(numStatsCols(size))
+        , min(n_stats_cols)
+        , max(n_stats_cols)
+        , null_count(n_stats_cols, 0)
+        , partition_index(partition_index_)
     {
         assert(size > 0);
     }
@@ -115,8 +152,8 @@ struct DeltaStats
     {
         assert(chunk.getNumRows() > 0);
         const auto & columns = chunk.getColumns();
-        assert(columns.size() == min.size() + partition_index.size());
-        for (size_t i = 0, col = 0; col < columns.size(); ++col)
+        assert(columns.size() - partition_index.size() >= n_stats_cols);
+        for (size_t i = 0, col = 0; i < n_stats_cols && col < columns.size(); ++col)
         {
             if (partition_index.contains(col))
                 continue;
@@ -131,10 +168,7 @@ struct DeltaStats
             this->null_count[i] += null_count;
 
             DB::Field min_value, max_value;
-            if (const auto * column_nullable = typeid_cast<const DB::ColumnNullable *>(column.get()))
-                column_nullable->getExtremesNullLast(min_value, max_value);
-            else
-                column->getExtremes(min_value, max_value);
+            column->getExtremes(min_value, max_value);
 
             assert(min[i].isNull() || min_value.getType() == min[i].getType());
             assert(max[i].isNull() || max_value.getType() == max[i].getType());
@@ -146,8 +180,8 @@ struct DeltaStats
             }
             else
             {
-                min[i] = applyVisitor(DB::FieldVisitorAccurateLess(), min[i], min_value) ? min[i] : min_value;
-                max[i] = applyVisitor(DB::FieldVisitorAccurateLess(), max[i], max_value) ? max_value : max[i];
+                min[i] = accurateLess(min[i], min_value) ? min[i] : min_value;
+                max[i] = accurateLess(max[i], max_value) ? max_value : max[i];
             }
             ++i;
         }
@@ -157,10 +191,10 @@ struct DeltaStats
 
     void merge(const DeltaStats & right)
     {
-        assert(min.size() == right.min.size());
+        assert(n_stats_cols == right.n_stats_cols);
         assert(partition_index == right.partition_index);
 
-        for (size_t i = 0; i < min.size(); ++i)
+        for (size_t i = 0; i < n_stats_cols; ++i)
         {
             null_count[i] += right.null_count[i];
             min[i] = std::min(min[i], right.min[i]);
@@ -244,9 +278,8 @@ public:
     void collectStats(const String & filename, const String & partition_dir, const DeltaStats & stats) const
     {
         const std::string & partition = partition_dir.empty() ? WriteStatsBase::NO_PARTITION_ID : partition_dir;
-        size_t columnSize = stats.min.size();
+        size_t columnSize = stats.n_stats_cols;
         assert(columns_.size() == stats_column_start + columnSize * 3);
-        assert(stats.min.size() == stats.max.size() && stats.min.size() == stats.null_count.size());
 
         columns_[ColumnIndex::filename]->insertData(filename.c_str(), filename.size());
         columns_[partition_id]->insertData(partition.c_str(), partition.size());
@@ -371,14 +404,25 @@ protected:
     }
     void onFinish() override
     {
-        if (output_format_) [[unlikely]]
+        if (output_format_)
         {
             output_format_->finalizeOutput();
+            /// We need close reset output_format_ here before return to spark, because the file is closed in ~WriteBufferFromHDFSImpl().
+            /// So that Spark Commit protocol can move the file safely.
+            output_format_.reset();
             assert(delta_stats_.row_count > 0);
             if (stats_)
                 stats_->collectStats(relative_path_, partition_id_, delta_stats_);
         }
     }
+    void onCancel() noexcept override
+    {
+        if (output_format_)
+        {
+            output_format_->cancel();
+            output_format_.reset();
+        }
+    }
 };
 
 class SparkPartitionedBaseSink : public DB::PartitionedSink
@@ -402,17 +446,23 @@ public:
         for (const auto & column : partition_columns)
         {
             // partition_column=
-            std::string key = add_slash ? fmt::format("/{}=", column) : fmt::format("{}=", column);
+            auto column_name = std::make_shared<DB::ASTLiteral>(column);
+            auto escaped_name = makeASTFunction("sparkPartitionEscape", DB::ASTs{column_name});
+            if (add_slash)
+                arguments.emplace_back(std::make_shared<DB::ASTLiteral>("/"));
             add_slash = true;
-            arguments.emplace_back(std::make_shared<DB::ASTLiteral>(key));
+            arguments.emplace_back(escaped_name);
+            arguments.emplace_back(std::make_shared<DB::ASTLiteral>("="));
 
             // ifNull(toString(partition_column), DEFAULT_PARTITION_NAME)
             // FIXME if toString(partition_column) is empty
-            auto column_ast = std::make_shared<DB::ASTIdentifier>(column);
+            auto column_ast = makeASTFunction("toString", DB::ASTs{std::make_shared<DB::ASTIdentifier>(column)});
+            auto escaped_value = makeASTFunction("sparkPartitionEscape", DB::ASTs{column_ast});
             DB::ASTs if_null_args{
-                makeASTFunction("toString", DB::ASTs{column_ast}), std::make_shared<DB::ASTLiteral>(DEFAULT_PARTITION_NAME)};
+                makeASTFunction("toString", DB::ASTs{escaped_value}), std::make_shared<DB::ASTLiteral>(DEFAULT_PARTITION_NAME)};
             arguments.emplace_back(makeASTFunction("ifNull", std::move(if_null_args)));
         }
+
         if (isBucketedWrite(input_header))
         {
             DB::ASTs args{std::make_shared<DB::ASTLiteral>("%05d"), std::make_shared<DB::ASTIdentifier>(BUCKET_COLUMN_NAME)};
