@@ -17,15 +17,14 @@
 #include "ParquetOutputFormatFile.h"
 
 #if USE_PARQUET
-#    include <memory>
-#    include <string>
+#include <memory>
+#include <string>
 
-#    include <Formats/FormatFactory.h>
-#    include <Formats/FormatSettings.h>
-#    include <Processors/Formats/Impl/ArrowBufferedStreams.h>
-#    include <Processors/Formats/Impl/CHColumnToArrowColumn.h>
-#    include <Processors/Formats/Impl/ParquetBlockOutputFormat.h>
-#    include <parquet/arrow/writer.h>
+#include <Formats/FormatFactory.h>
+#include <Formats/FormatSettings.h>
+#include <Processors/Formats/Impl/ArrowBufferedStreams.h>
+#include <Processors/Formats/Impl/ParquetBlockOutputFormat.h>
+#include <Processors/Port.h>
 
 namespace local_engine
 {
