@@ -25,23 +25,25 @@
 namespace local_engine
 {
 
-JSONFormatFile::JSONFormatFile(DB::ContextPtr context_, const substrait::ReadRel::LocalFiles::FileOrFiles & file_info_, ReadBufferBuilderPtr read_buffer_builder_)
-    :FormatFile(context_, file_info_, read_buffer_builder_) {}
+JSONFormatFile::JSONFormatFile(
+    DB::ContextPtr context_, const substrait::ReadRel::LocalFiles::FileOrFiles & file_info_, ReadBufferBuilderPtr read_buffer_builder_)
+    : FormatFile(context_, file_info_, read_buffer_builder_)
+{
+}
 
 FormatFile::InputFormatPtr JSONFormatFile::createInputFormat(const DB::Block & header)
 {
-    auto res = std::make_shared<FormatFile::InputFormat>();
-    res->read_buffer = read_buffer_builder->buildWithCompressionWrapper(file_info);
+    auto read_buffer = read_buffer_builder->buildWithCompressionWrapper(file_info);
 
     DB::FormatSettings format_settings = DB::getFormatSettings(context);
     format_settings.with_names_use_header = true;
     format_settings.skip_unknown_fields = true;
     size_t max_block_size = file_info.json().max_block_size();
     DB::RowInputFormatParams in_params = {max_block_size};
-    std::shared_ptr<DB::JSONEachRowRowInputFormat> json_input_format =
-        std::make_shared<DB::JSONEachRowRowInputFormat>(*(res->read_buffer), header, in_params, format_settings, false);
-    res->input = json_input_format;
-    return res;
+    std::shared_ptr<DB::JSONEachRowRowInputFormat> json_input_format
+        = std::make_shared<DB::JSONEachRowRowInputFormat>(*read_buffer, header, in_params, format_settings, false);
+
+    return std::make_shared<InputFormat>(std::move(read_buffer), json_input_format);
 }
 
 }
