@@ -110,17 +110,19 @@ void ListenableMemoryAllocator::updateUsage(int64_t size) {
   usedBytes_ += size;
   while (true) {
     int64_t savedPeakBytes = peakBytes_;
-    if (usedBytes_ <= savedPeakBytes) {
+    int64_t savedUsedBytes = usedBytes_;
+    if (savedUsedBytes <= savedPeakBytes) {
       break;
     }
     // usedBytes_ > savedPeakBytes, update peak
-    if (peakBytes_.compare_exchange_weak(savedPeakBytes, usedBytes_)) {
+    if (peakBytes_.compare_exchange_weak(savedPeakBytes, savedUsedBytes)) {
       break;
     }
   }
 }
 
 bool StdMemoryAllocator::allocate(int64_t size, void** out) {
+  GLUTEN_CHECK(size >= 0, "size is less than 0");
   *out = std::malloc(size);
   if (*out == nullptr) {
     return false;
@@ -130,6 +132,8 @@ bool StdMemoryAllocator::allocate(int64_t size, void** out) {
 }
 
 bool StdMemoryAllocator::allocateZeroFilled(int64_t nmemb, int64_t size, void** out) {
+  GLUTEN_CHECK(nmemb >= 0, "nmemb is less than 0");
+  GLUTEN_CHECK(size >= 0, "size is less than 0");
   *out = std::calloc(nmemb, size);
   if (*out == nullptr) {
     return false;
@@ -139,6 +143,7 @@ bool StdMemoryAllocator::allocateZeroFilled(int64_t nmemb, int64_t size, void**
 }
 
 bool StdMemoryAllocator::allocateAligned(uint64_t alignment, int64_t size, void** out) {
+  GLUTEN_CHECK(size >= 0, "size is less than 0");
   *out = aligned_alloc(alignment, size);
   if (*out == nullptr) {
     return false;
@@ -157,11 +162,12 @@ bool StdMemoryAllocator::reallocate(void* p, int64_t size, int64_t newSize, void
 }
 
 bool StdMemoryAllocator::reallocateAligned(void* p, uint64_t alignment, int64_t size, int64_t newSize, void** out) {
+  GLUTEN_CHECK(p != nullptr, "reallocate with nullptr");
   if (newSize <= 0) {
     return false;
   }
   if (newSize <= size) {
-    auto aligned = ROUND_TO_LINE(newSize, alignment);
+    auto aligned = ROUND_TO_LINE(static_cast<uint64_t>(newSize), alignment);
     if (aligned <= size) {
       // shrink-to-fit
       return reallocate(p, size, aligned, out);
@@ -179,6 +185,7 @@ bool StdMemoryAllocator::reallocateAligned(void* p, uint64_t alignment, int64_t
 }
 
 bool StdMemoryAllocator::free(void* p, int64_t size) {
+  GLUTEN_CHECK(p != nullptr, "free with nullptr");
   std::free(p);
   bytes_ -= size;
   return true;
