@@ -24,8 +24,8 @@
 
 #include "shuffle/Options.h"
 #include "shuffle/Utils.h"
+#include "utils/Exception.h"
 #include "utils/Timer.h"
-#include "utils/exception.h"
 
 namespace gluten {
 namespace {
@@ -50,16 +50,14 @@ T* advance(uint8_t** dst) {
   return ptr;
 }
 
-arrow::Result<std::pair<uint8_t, uint32_t>> readTypeAndRows(arrow::io::InputStream* inputStream) {
+arrow::Result<uint8_t> readType(arrow::io::InputStream* inputStream) {
   uint8_t type;
-  uint32_t numRows;
   ARROW_ASSIGN_OR_RAISE(auto bytes, inputStream->Read(sizeof(Payload::Type), &type));
   if (bytes == 0) {
     // Reach EOS.
-    return std::make_pair(0, 0);
+    return 0;
   }
-  RETURN_NOT_OK(inputStream->Read(sizeof(uint32_t), &numRows));
-  return std::make_pair(type, numRows);
+  return type;
 }
 
 arrow::Result<int64_t> compressBuffer(
@@ -120,13 +118,16 @@ arrow::Status compressAndFlush(
   return arrow::Status::OK();
 }
 
-arrow::Result<std::shared_ptr<arrow::Buffer>> readUncompressedBuffer(arrow::io::InputStream* inputStream) {
+arrow::Result<std::shared_ptr<arrow::Buffer>>
+readUncompressedBuffer(arrow::io::InputStream* inputStream, arrow::MemoryPool* pool, int64_t& deserializedTime) {
+  ScopedTimer timer(&deserializedTime);
   int64_t bufferLength;
   RETURN_NOT_OK(inputStream->Read(sizeof(int64_t), &bufferLength));
   if (bufferLength == kNullBuffer) {
     return nullptr;
   }
-  ARROW_ASSIGN_OR_RAISE(auto buffer, inputStream->Read(bufferLength));
+  ARROW_ASSIGN_OR_RAISE(auto buffer, arrow::AllocateResizableBuffer(bufferLength, pool));
+  RETURN_NOT_OK(inputStream->Read(bufferLength, buffer->mutable_data()));
   return buffer;
 }
 
@@ -134,7 +135,9 @@ arrow::Result<std::shared_ptr<arrow::Buffer>> readCompressedBuffer(
     arrow::io::InputStream* inputStream,
     const std::shared_ptr<arrow::util::Codec>& codec,
     arrow::MemoryPool* pool,
+    int64_t& deserializeTime,
     int64_t& decompressTime) {
+  ScopedTimer timer(&deserializeTime);
   int64_t compressedLength;
   RETURN_NOT_OK(inputStream->Read(sizeof(int64_t), &compressedLength));
   if (compressedLength == kNullBuffer) {
@@ -148,16 +151,15 @@ arrow::Result<std::shared_ptr<arrow::Buffer>> readCompressedBuffer(
   RETURN_NOT_OK(inputStream->Read(sizeof(int64_t), &uncompressedLength));
   if (compressedLength == kUncompressedBuffer) {
     ARROW_ASSIGN_OR_RAISE(auto uncompressed, arrow::AllocateResizableBuffer(uncompressedLength, pool));
-    RETURN_NOT_OK(inputStream->Read(uncompressedLength, const_cast<uint8_t*>(uncompressed->data())));
+    RETURN_NOT_OK(inputStream->Read(uncompressedLength, uncompressed->mutable_data()));
     return uncompressed;
   }
-  ARROW_ASSIGN_OR_RAISE(auto compressed, arrow::AllocateBuffer(compressedLength, pool));
-  RETURN_NOT_OK(inputStream->Read(compressedLength, const_cast<uint8_t*>(compressed->data())));
+  ARROW_ASSIGN_OR_RAISE(auto compressed, arrow::AllocateResizableBuffer(compressedLength, pool));
+  RETURN_NOT_OK(inputStream->Read(compressedLength, compressed->mutable_data()));
 
-  ScopedTimer timer(&decompressTime);
+  timer.switchTo(&decompressTime);
   ARROW_ASSIGN_OR_RAISE(auto output, arrow::AllocateResizableBuffer(uncompressedLength, pool));
-  RETURN_NOT_OK(codec->Decompress(
-      compressedLength, compressed->data(), uncompressedLength, const_cast<uint8_t*>(output->data())));
+  RETURN_NOT_OK(codec->Decompress(compressedLength, compressed->data(), uncompressedLength, output->mutable_data()));
   return output;
 }
 
@@ -186,29 +188,30 @@ arrow::Result<std::unique_ptr<BlockPayload>> BlockPayload::fromBuffers(
     std::vector<std::shared_ptr<arrow::Buffer>> buffers,
     const std::vector<bool>* isValidityBuffer,
     arrow::MemoryPool* pool,
-    arrow::util::Codec* codec) {
+    arrow::util::Codec* codec,
+    std::shared_ptr<arrow::Buffer> compressed) {
   if (payloadType == Payload::Type::kCompressed) {
     Timer compressionTime;
     compressionTime.start();
     // Compress.
-    // Compressed buffer layout: | buffer1 compressedLength | buffer1 uncompressedLength | buffer1 | ...
-    const auto metadataLength = sizeof(int64_t) * 2 * buffers.size();
-    int64_t totalCompressedLength =
-        std::accumulate(buffers.begin(), buffers.end(), 0LL, [&](auto sum, const auto& buffer) {
-          if (!buffer) {
-            return sum;
-          }
-          return sum + codec->MaxCompressedLen(buffer->size(), buffer->data());
-        });
-    const auto maxCompressedLength = metadataLength + totalCompressedLength;
-    ARROW_ASSIGN_OR_RAISE(
-        std::shared_ptr<arrow::ResizableBuffer> compressed, arrow::AllocateResizableBuffer(maxCompressedLength, pool));
+    auto maxLength = maxCompressedLength(buffers, codec);
+    std::shared_ptr<arrow::Buffer> compressedBuffer;
+    uint8_t* output;
+    if (compressed) {
+      ARROW_RETURN_IF(
+          compressed->size() < maxLength,
+          arrow::Status::Invalid(
+              "Compressed buffer length < maxCompressedLength. (", compressed->size(), " vs ", maxLength, ")"));
+      output = const_cast<uint8_t*>(compressed->data());
+    } else {
+      ARROW_ASSIGN_OR_RAISE(compressedBuffer, arrow::AllocateResizableBuffer(maxLength, pool));
+      output = compressedBuffer->mutable_data();
+    }
 
-    auto output = compressed->mutable_data();
     int64_t actualLength = 0;
     // Compress buffers one by one.
     for (auto& buffer : buffers) {
-      auto availableLength = maxCompressedLength - actualLength;
+      auto availableLength = maxLength - actualLength;
       // Release buffer after compression.
       ARROW_ASSIGN_OR_RAISE(auto compressedSize, compressBuffer(std::move(buffer), output, availableLength, codec));
       output += compressedSize;
@@ -216,15 +219,14 @@ arrow::Result<std::unique_ptr<BlockPayload>> BlockPayload::fromBuffers(
     }
 
     ARROW_RETURN_IF(actualLength < 0, arrow::Status::Invalid("Writing compressed buffer out of bound."));
-    RETURN_NOT_OK(compressed->Resize(actualLength));
+    if (compressed) {
+      compressedBuffer = std::make_shared<arrow::Buffer>(compressed->data(), actualLength);
+    } else {
+      RETURN_NOT_OK(std::dynamic_pointer_cast<arrow::ResizableBuffer>(compressedBuffer)->Resize(actualLength));
+    }
     compressionTime.stop();
-    auto payload = std::unique_ptr<BlockPayload>(new BlockPayload(
-        Type::kCompressed,
-        numRows,
-        std::vector<std::shared_ptr<arrow::Buffer>>{compressed},
-        isValidityBuffer,
-        pool,
-        codec));
+    auto payload = std::unique_ptr<BlockPayload>(
+        new BlockPayload(Type::kCompressed, numRows, {compressedBuffer}, isValidityBuffer, pool, codec));
     payload->setCompressionTime(compressionTime.realTimeUsed());
     return payload;
   }
@@ -238,6 +240,8 @@ arrow::Status BlockPayload::serialize(arrow::io::OutputStream* outputStream) {
       ScopedTimer timer(&writeTime_);
       RETURN_NOT_OK(outputStream->Write(&kUncompressedType, sizeof(Type)));
       RETURN_NOT_OK(outputStream->Write(&numRows_, sizeof(uint32_t)));
+      uint32_t numBuffers = buffers_.size();
+      RETURN_NOT_OK(outputStream->Write(&numBuffers, sizeof(uint32_t)));
       for (auto& buffer : buffers_) {
         if (!buffer) {
           RETURN_NOT_OK(outputStream->Write(&kNullBuffer, sizeof(int64_t)));
@@ -255,6 +259,8 @@ arrow::Status BlockPayload::serialize(arrow::io::OutputStream* outputStream) {
         ScopedTimer timer(&writeTime_);
         RETURN_NOT_OK(outputStream->Write(&kCompressedType, sizeof(Type)));
         RETURN_NOT_OK(outputStream->Write(&numRows_, sizeof(uint32_t)));
+        uint32_t numBuffers = buffers_.size();
+        RETURN_NOT_OK(outputStream->Write(&numBuffers, sizeof(uint32_t)));
       }
       for (auto& buffer : buffers_) {
         RETURN_NOT_OK(compressAndFlush(std::move(buffer), outputStream, codec_, pool_, compressTime_, writeTime_));
@@ -264,6 +270,8 @@ arrow::Status BlockPayload::serialize(arrow::io::OutputStream* outputStream) {
       ScopedTimer timer(&writeTime_);
       RETURN_NOT_OK(outputStream->Write(&kCompressedType, sizeof(Type)));
       RETURN_NOT_OK(outputStream->Write(&numRows_, sizeof(uint32_t)));
+      uint32_t buffers = numBuffers();
+      RETURN_NOT_OK(outputStream->Write(&buffers, sizeof(uint32_t)));
       RETURN_NOT_OK(outputStream->Write(std::move(buffers_[0])));
     } break;
     case Type::kRaw: {
@@ -279,68 +287,42 @@ arrow::Result<std::shared_ptr<arrow::Buffer>> BlockPayload::readBufferAt(uint32_
   if (type_ == Type::kCompressed) {
     return arrow::Status::Invalid("Cannot read buffer from compressed BlockPayload.");
   }
+  if (type_ == Type::kRaw && pos != 0) {
+    return arrow::Status::Invalid("Read buffer pos from raw should only be 0, but got " + std::to_string(pos));
+  }
   return std::move(buffers_[pos]);
 }
 
 arrow::Result<std::vector<std::shared_ptr<arrow::Buffer>>> BlockPayload::deserialize(
     arrow::io::InputStream* inputStream,
-    const std::shared_ptr<arrow::Schema>& schema,
     const std::shared_ptr<arrow::util::Codec>& codec,
     arrow::MemoryPool* pool,
     uint32_t& numRows,
+    int64_t& deserializeTime,
     int64_t& decompressTime) {
+  auto timer = std::make_unique<ScopedTimer>(&deserializeTime);
   static const std::vector<std::shared_ptr<arrow::Buffer>> kEmptyBuffers{};
-  ARROW_ASSIGN_OR_RAISE(auto typeAndRows, readTypeAndRows(inputStream));
-  if (typeAndRows.first == 0) {
+  ARROW_ASSIGN_OR_RAISE(auto type, readType(inputStream));
+  if (type == 0) {
     numRows = 0;
     return kEmptyBuffers;
   }
-  numRows = typeAndRows.second;
-  auto fields = schema->fields();
+  RETURN_NOT_OK(inputStream->Read(sizeof(uint32_t), &numRows));
+  uint32_t numBuffers;
+  RETURN_NOT_OK(inputStream->Read(sizeof(uint32_t), &numBuffers));
+  timer.reset();
 
-  auto isCompressionEnabled = typeAndRows.first == Type::kCompressed;
-  auto readBuffer = [&]() {
-    if (isCompressionEnabled) {
-      return readCompressedBuffer(inputStream, codec, pool, decompressTime);
-    } else {
-      return readUncompressedBuffer(inputStream);
-    }
-  };
-
-  bool hasComplexDataType = false;
+  bool isCompressionEnabled = type == Type::kCompressed;
   std::vector<std::shared_ptr<arrow::Buffer>> buffers;
-  for (const auto& field : fields) {
-    auto fieldType = field->type()->id();
-    switch (fieldType) {
-      case arrow::BinaryType::type_id:
-      case arrow::StringType::type_id: {
-        buffers.emplace_back();
-        ARROW_ASSIGN_OR_RAISE(buffers.back(), readBuffer());
-        buffers.emplace_back();
-        ARROW_ASSIGN_OR_RAISE(buffers.back(), readBuffer());
-        buffers.emplace_back();
-        ARROW_ASSIGN_OR_RAISE(buffers.back(), readBuffer());
-        break;
-      }
-      case arrow::StructType::type_id:
-      case arrow::MapType::type_id:
-      case arrow::ListType::type_id: {
-        hasComplexDataType = true;
-      } break;
-      case arrow::NullType::type_id:
-        break;
-      default: {
-        buffers.emplace_back();
-        ARROW_ASSIGN_OR_RAISE(buffers.back(), readBuffer());
-        buffers.emplace_back();
-        ARROW_ASSIGN_OR_RAISE(buffers.back(), readBuffer());
-        break;
-      }
-    }
-  }
-  if (hasComplexDataType) {
+  buffers.reserve(numBuffers);
+  for (auto i = 0; i < numBuffers; ++i) {
     buffers.emplace_back();
-    ARROW_ASSIGN_OR_RAISE(buffers.back(), readBuffer());
+    if (isCompressionEnabled) {
+      ARROW_ASSIGN_OR_RAISE(
+          buffers.back(), readCompressedBuffer(inputStream, codec, pool, deserializeTime, decompressTime));
+    } else {
+      ARROW_ASSIGN_OR_RAISE(buffers.back(), readUncompressedBuffer(inputStream, pool, deserializeTime));
+    }
   }
   return buffers;
 }
@@ -349,6 +331,25 @@ void BlockPayload::setCompressionTime(int64_t compressionTime) {
   compressTime_ = compressionTime;
 }
 
+int64_t BlockPayload::rawSize() {
+  return getBufferSize(buffers_);
+}
+
+int64_t BlockPayload::maxCompressedLength(
+    const std::vector<std::shared_ptr<arrow::Buffer>>& buffers,
+    arrow::util::Codec* codec) {
+  // Compressed buffer layout: | buffer1 compressedLength | buffer1 uncompressedLength | buffer1 | ...
+  const auto metadataLength = sizeof(int64_t) * 2 * buffers.size();
+  int64_t totalCompressedLength =
+      std::accumulate(buffers.begin(), buffers.end(), 0LL, [&](auto sum, const auto& buffer) {
+        if (!buffer) {
+          return sum;
+        }
+        return sum + codec->MaxCompressedLen(buffer->size(), buffer->data());
+      });
+  return metadataLength + totalCompressedLength;
+}
+
 arrow::Result<std::unique_ptr<InMemoryPayload>> InMemoryPayload::merge(
     std::unique_ptr<InMemoryPayload> source,
     std::unique_ptr<InMemoryPayload> append,
@@ -424,9 +425,13 @@ arrow::Result<std::unique_ptr<InMemoryPayload>> InMemoryPayload::merge(
   return std::make_unique<InMemoryPayload>(mergedRows, isValidityBuffer, std::move(merged));
 }
 
-arrow::Result<std::unique_ptr<BlockPayload>>
-InMemoryPayload::toBlockPayload(Payload::Type payloadType, arrow::MemoryPool* pool, arrow::util::Codec* codec) {
-  return BlockPayload::fromBuffers(payloadType, numRows_, std::move(buffers_), isValidityBuffer_, pool, codec);
+arrow::Result<std::unique_ptr<BlockPayload>> InMemoryPayload::toBlockPayload(
+    Payload::Type payloadType,
+    arrow::MemoryPool* pool,
+    arrow::util::Codec* codec,
+    std::shared_ptr<arrow::Buffer> compressed) {
+  return BlockPayload::fromBuffers(
+      payloadType, numRows_, std::move(buffers_), isValidityBuffer_, pool, codec, std::move(compressed));
 }
 
 arrow::Status InMemoryPayload::serialize(arrow::io::OutputStream* outputStream) {
@@ -437,10 +442,6 @@ arrow::Result<std::shared_ptr<arrow::Buffer>> InMemoryPayload::readBufferAt(uint
   return std::move(buffers_[index]);
 }
 
-int64_t InMemoryPayload::getBufferSize() const {
-  return gluten::getBufferSize(buffers_);
-}
-
 arrow::Status InMemoryPayload::copyBuffers(arrow::MemoryPool* pool) {
   for (auto& buffer : buffers_) {
     if (!buffer) {
@@ -457,6 +458,10 @@ arrow::Status InMemoryPayload::copyBuffers(arrow::MemoryPool* pool) {
   return arrow::Status::OK();
 }
 
+int64_t InMemoryPayload::rawSize() {
+  return getBufferSize(buffers_);
+}
+
 UncompressedDiskBlockPayload::UncompressedDiskBlockPayload(
     Type type,
     uint32_t numRows,
@@ -490,13 +495,23 @@ arrow::Status UncompressedDiskBlockPayload::serialize(arrow::io::OutputStream* o
       arrow::Status::Invalid(
           "Invalid payload type: " + std::to_string(type_) +
           ", should be either Payload::kUncompressed or Payload::kToBeCompressed"));
-  ARROW_ASSIGN_OR_RAISE(auto startPos, inputStream_->Tell());
-  auto typeAndRows = readTypeAndRows(inputStream_);
-  // Discard type and rows.
-  RETURN_NOT_OK(typeAndRows.status());
   RETURN_NOT_OK(outputStream->Write(&kCompressedType, sizeof(kCompressedType)));
   RETURN_NOT_OK(outputStream->Write(&numRows_, sizeof(uint32_t)));
-  auto readPos = startPos + sizeof(kUncompressedType) + sizeof(uint32_t);
+
+  ARROW_ASSIGN_OR_RAISE(auto startPos, inputStream_->Tell());
+
+  // Discard original type and rows.
+  Payload::Type type;
+  uint32_t numRows;
+  ARROW_ASSIGN_OR_RAISE(auto bytes, inputStream_->Read(sizeof(Payload::Type), &type));
+  ARROW_ASSIGN_OR_RAISE(bytes, inputStream_->Read(sizeof(uint32_t), &numRows));
+  uint32_t numBuffers = 0;
+  ARROW_ASSIGN_OR_RAISE(bytes, inputStream_->Read(sizeof(uint32_t), &numBuffers));
+  ARROW_RETURN_IF(bytes == 0 || numBuffers == 0, arrow::Status::Invalid("Cannot serialize payload with 0 buffers."));
+  RETURN_NOT_OK(outputStream->Write(&numBuffers, sizeof(uint32_t)));
+
+  // Advance Payload::Type, rows and numBuffers.
+  auto readPos = startPos + sizeof(Payload::Type) + sizeof(uint32_t) + sizeof(uint32_t);
   while (readPos - startPos < rawSize_) {
     ARROW_ASSIGN_OR_RAISE(auto uncompressed, readUncompressedBuffer());
     ARROW_ASSIGN_OR_RAISE(readPos, inputStream_->Tell());
@@ -520,11 +535,15 @@ arrow::Result<std::shared_ptr<arrow::Buffer>> UncompressedDiskBlockPayload::read
   return buffer;
 }
 
+int64_t UncompressedDiskBlockPayload::rawSize() {
+  return rawSize_;
+}
+
 CompressedDiskBlockPayload::CompressedDiskBlockPayload(
     uint32_t numRows,
     const std::vector<bool>* isValidityBuffer,
     arrow::io::InputStream*& inputStream,
-    uint64_t rawSize,
+    int64_t rawSize,
     arrow::MemoryPool* /* pool */)
     : Payload(Type::kCompressed, numRows, isValidityBuffer), inputStream_(inputStream), rawSize_(rawSize) {}
 
@@ -540,4 +559,8 @@ arrow::Status CompressedDiskBlockPayload::serialize(arrow::io::OutputStream* out
 arrow::Result<std::shared_ptr<arrow::Buffer>> CompressedDiskBlockPayload::readBufferAt(uint32_t index) {
   return arrow::Status::Invalid("Cannot read buffer from CompressedDiskBlockPayload.");
 }
+
+int64_t CompressedDiskBlockPayload::rawSize() {
+  return rawSize_;
+}
 } // namespace gluten
