@@ -40,7 +40,7 @@ std::unordered_map<std::string, std::string> defaultConf() {
 }
 
 void initVeloxBackend(std::unordered_map<std::string, std::string>& conf) {
-  gluten::VeloxBackend::create(conf);
+  gluten::VeloxBackend::create(AllocationListener::noop(), conf);
 }
 
 void initVeloxBackend() {
@@ -123,6 +123,8 @@ bool endsWith(const std::string& data, const std::string& suffix) {
 }
 
 void setCpu(uint32_t cpuIndex) {
+#ifdef __APPLE__
+#else
   static const auto kTotalCores = std::thread::hardware_concurrency();
   cpuIndex = cpuIndex % kTotalCores;
   cpu_set_t cs;
@@ -132,6 +134,7 @@ void setCpu(uint32_t cpuIndex) {
     LOG(WARNING) << "Error binding CPU " << std::to_string(cpuIndex);
     std::exit(EXIT_FAILURE);
   }
+#endif
 }
 
 void BenchmarkAllocationListener::allocationChanged(int64_t diff) {
