@@ -58,16 +58,8 @@ const std::unordered_set<std::string> kRegexFunctions = {
     "regexp_replace",
     "rlike"};
 
-const std::unordered_set<std::string> kBlackList = {
-    "split_part",
-    "factorial",
-    "from_json",
-    "json_array_length",
-    "trunc",
-    "sequence",
-    "approx_percentile",
-    "get_array_struct_fields",
-    "map_from_arrays"};
+const std::unordered_set<std::string> kBlackList =
+    {"split_part", "factorial", "trunc", "sequence", "approx_percentile", "get_array_struct_fields", "map_from_arrays"};
 } // namespace
 
 bool SubstraitToVeloxPlanValidator::parseVeloxType(
@@ -90,7 +82,7 @@ bool SubstraitToVeloxPlanValidator::parseVeloxType(
 }
 
 bool SubstraitToVeloxPlanValidator::flattenSingleLevel(const TypePtr& type, std::vector<TypePtr>& out) {
-  if (type->kind() != TypeKind::ROW) {
+  if (!type->isRow()) {
     LOG_VALIDATION_MSG("Type is not a RowType.");
     return false;
   }
@@ -106,7 +98,7 @@ bool SubstraitToVeloxPlanValidator::flattenSingleLevel(const TypePtr& type, std:
 }
 
 bool SubstraitToVeloxPlanValidator::flattenDualLevel(const TypePtr& type, std::vector<std::vector<TypePtr>>& out) {
-  if (type->kind() != TypeKind::ROW) {
+  if (!type->isRow()) {
     LOG_VALIDATION_MSG("Type is not a RowType.");
     return false;
   }
@@ -213,9 +205,11 @@ bool SubstraitToVeloxPlanValidator::validateScalarFunction(
 
   if (name == "round") {
     return validateRound(scalarFunction, inputType);
-  } else if (name == "extract") {
+  }
+  if (name == "extract") {
     return validateExtractExpr(params);
-  } else if (name == "concat") {
+  }
+  if (name == "concat") {
     for (const auto& type : types) {
       if (type.find("struct") != std::string::npos || type.find("map") != std::string::npos ||
           type.find("list") != std::string::npos) {
@@ -238,24 +232,58 @@ bool SubstraitToVeloxPlanValidator::validateScalarFunction(
   return true;
 }
 
-bool SubstraitToVeloxPlanValidator::validateLiteral(
-    const ::substrait::Expression_Literal& literal,
-    const RowTypePtr& inputType) {
-  if (literal.has_list()) {
-    for (auto child : literal.list().values()) {
-      if (!validateLiteral(child, inputType)) {
-        // the error msg has been set, so do not need to set it again.
-        return false;
-      }
-    }
-  } else if (literal.has_map()) {
-    for (auto child : literal.map().key_values()) {
-      if (!validateLiteral(child.key(), inputType) || !validateLiteral(child.value(), inputType)) {
-        // the error msg has been set, so do not need to set it again.
-        return false;
-      }
-    }
+bool SubstraitToVeloxPlanValidator::isAllowedCast(const TypePtr& fromType, const TypePtr& toType) {
+  // Currently cast is not allowed for various categories, code has a bunch of rules
+  // which define the cast categories and if we should offload to velox. Currently,
+  // the following categories are denied.
+  //
+  // 1. from/to isIntervalYearMonth is not allowed.
+  // 2. Date to most categories except few supported types is not allowed.
+  // 3. Timestamp to most categories except few supported types is not allowed.
+  // 4. Certain complex types are not allowed.
+
+  // Don't support isIntervalYearMonth.
+  if (fromType->isIntervalYearMonth() || toType->isIntervalYearMonth()) {
+    return false;
   }
+
+  // Limited support for DATE to X.
+  if (fromType->isDate() && !toType->isTimestamp() && !toType->isVarchar()) {
+    return false;
+  }
+
+  // Limited support for Timestamp to X.
+  if (fromType->isTimestamp() && !(toType->isDate() || toType->isVarchar())) {
+    return false;
+  }
+
+  // Limited support for X to Timestamp.
+  if (toType->isTimestamp()) {
+    if (fromType->isDecimal()) {
+      return false;
+    }
+    if (fromType->isDate()) {
+      return true;
+    }
+    if (fromType->isVarchar()) {
+      return true;
+    }
+    if (fromType->isTinyint() || fromType->isSmallint() || fromType->isInteger() || fromType->isBigint() ||
+        fromType->isDouble() || fromType->isReal()) {
+      return true;
+    }
+    return false;
+  }
+
+  // Limited support for Complex types.
+  if (fromType->isArray() || fromType->isMap() || fromType->isRow()) {
+    return false;
+  }
+
+  if (fromType->isVarbinary() && !toType->isVarchar()) {
+    return false;
+  }
+
   return true;
 }
 
@@ -269,47 +297,12 @@ bool SubstraitToVeloxPlanValidator::validateCast(
   const auto& toType = SubstraitParser::parseType(castExpr.type());
   core::TypedExprPtr input = exprConverter_->toVeloxExpr(castExpr.input(), inputType);
 
-  // Only support cast from date to timestamp
-  if (toType->kind() == TypeKind::TIMESTAMP && !input->type()->isDate()) {
-    LOG_VALIDATION_MSG(
-        "Casting from " + input->type()->toString() + " to " + toType->toString() + " is not supported.");
-    return false;
+  if (SubstraitToVeloxPlanValidator::isAllowedCast(input->type(), toType)) {
+    return true;
   }
 
-  if (toType->isIntervalYearMonth()) {
-    LOG_VALIDATION_MSG("Casting to " + toType->toString() + " is not supported.");
-    return false;
-  }
-
-  // Casting from some types is not supported. See CastExpr::applyPeeled.
-  if (input->type()->isDate()) {
-    // Only support cast date to varchar & timestamp
-    if (toType->kind() != TypeKind::VARCHAR && toType->kind() != TypeKind::TIMESTAMP) {
-      LOG_VALIDATION_MSG("Casting from DATE to " + toType->toString() + " is not supported.");
-      return false;
-    }
-  } else if (input->type()->isIntervalYearMonth()) {
-    LOG_VALIDATION_MSG("Casting from INTERVAL_YEAR_MONTH is not supported.");
-    return false;
-  }
-  switch (input->type()->kind()) {
-    case TypeKind::ARRAY:
-    case TypeKind::MAP:
-    case TypeKind::ROW:
-    case TypeKind::VARBINARY:
-      LOG_VALIDATION_MSG("Invalid input type in casting: ARRAY/MAP/ROW/VARBINARY.");
-      return false;
-    case TypeKind::TIMESTAMP:
-      // Only support casting timestamp to date or varchar.
-      if (!toType->isDate() && toType->kind() != TypeKind::VARCHAR) {
-        LOG_VALIDATION_MSG(
-            "Casting from TIMESTAMP to " + toType->toString() + " is not supported or has incorrect result.");
-        return false;
-      }
-    default: {
-    }
-  }
-  return true;
+  LOG_VALIDATION_MSG("Casting from " + input->type()->toString() + " to " + toType->toString() + " is not supported.");
+  return false;
 }
 
 bool SubstraitToVeloxPlanValidator::validateIfThen(
@@ -334,9 +327,6 @@ bool SubstraitToVeloxPlanValidator::validateSingularOrList(
       LOG_VALIDATION_MSG("Option is expected as Literal.");
       return false;
     }
-    if (!validateLiteral(option.literal(), inputType)) {
-      return false;
-    }
   }
 
   return validateExpression(singularOrList.value(), inputType);
@@ -349,8 +339,6 @@ bool SubstraitToVeloxPlanValidator::validateExpression(
   switch (typeCase) {
     case ::substrait::Expression::RexTypeCase::kScalarFunction:
       return validateScalarFunction(expression.scalar_function(), inputType);
-    case ::substrait::Expression::RexTypeCase::kLiteral:
-      return validateLiteral(expression.literal(), inputType);
     case ::substrait::Expression::RexTypeCase::kCast:
       return validateCast(expression.cast(), inputType);
     case ::substrait::Expression::RexTypeCase::kIfThen:
@@ -565,7 +553,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::ExpandRel& expan
       if (rowType) {
         // Try to compile the expressions. If there is any unregistered
         // function or mismatched type, exception will be thrown.
-        exec::ExprSet exprSet(std::move(expressions), execCtx_);
+        exec::ExprSet exprSet(std::move(expressions), execCtx_.get());
       }
     } else {
       LOG_VALIDATION_MSG("Only SwitchingField is supported in ExpandRel.");
@@ -679,7 +667,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::WindowRel& windo
   }
   // Try to compile the expressions. If there is any unregistred funciton or
   // mismatched type, exception will be thrown.
-  exec::ExprSet exprSet(std::move(expressions), execCtx_);
+  exec::ExprSet exprSet(std::move(expressions), execCtx_.get());
 
   // Validate Sort expression
   const auto& sorts = windowRel.sorts();
@@ -702,7 +690,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::WindowRel& windo
         LOG_VALIDATION_MSG("in windowRel, the sorting key in Sort Operator only support field.");
         return false;
       }
-      exec::ExprSet exprSet1({std::move(expression)}, execCtx_);
+      exec::ExprSet exprSet1({std::move(expression)}, execCtx_.get());
     }
   }
 
@@ -745,13 +733,12 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::WindowGroupLimit
     if (exprField == nullptr) {
       LOG_VALIDATION_MSG("Only field is supported for partition key in Window Group Limit Operator!");
       return false;
-    } else {
-      expressions.emplace_back(expression);
     }
+    expressions.emplace_back(expression);
   }
   // Try to compile the expressions. If there is any unregistered function or
   // mismatched type, exception will be thrown.
-  exec::ExprSet exprSet(std::move(expressions), execCtx_);
+  exec::ExprSet exprSet(std::move(expressions), execCtx_.get());
   // Validate Sort expression
   const auto& sorts = windowGroupLimitRel.sorts();
   for (const auto& sort : sorts) {
@@ -773,7 +760,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::WindowGroupLimit
         LOG_VALIDATION_MSG("in windowGroupLimitRel, the sorting key in Sort Operator only support field.");
         return false;
       }
-      exec::ExprSet exprSet1({std::move(expression)}, execCtx_);
+      exec::ExprSet exprSet1({std::move(expression)}, execCtx_.get());
     }
   }
 
@@ -875,7 +862,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::SortRel& sortRel
         LOG_VALIDATION_MSG("in SortRel, the sorting key in Sort Operator only support field.");
         return false;
       }
-      exec::ExprSet exprSet({std::move(expression)}, execCtx_);
+      exec::ExprSet exprSet({std::move(expression)}, execCtx_.get());
     }
   }
 
@@ -922,7 +909,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::ProjectRel& proj
   }
   // Try to compile the expressions. If there is any unregistered function or
   // mismatched type, exception will be thrown.
-  exec::ExprSet exprSet(std::move(expressions), execCtx_);
+  exec::ExprSet exprSet(std::move(expressions), execCtx_.get());
   return true;
 }
 
@@ -961,7 +948,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::FilterRel& filte
   expressions.emplace_back(exprConverter_->toVeloxExpr(filterRel.condition(), rowType));
   // Try to compile the expressions. If there is any unregistered function
   // or mismatched type, exception will be thrown.
-  exec::ExprSet exprSet(std::move(expressions), execCtx_);
+  exec::ExprSet exprSet(std::move(expressions), execCtx_.get());
   return true;
 }
 
@@ -1035,7 +1022,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::JoinRel& joinRel
 
   if (joinRel.has_post_join_filter()) {
     auto expression = exprConverter_->toVeloxExpr(joinRel.post_join_filter(), rowType);
-    exec::ExprSet exprSet({std::move(expression)}, execCtx_);
+    exec::ExprSet exprSet({std::move(expression)}, execCtx_.get());
   }
   return true;
 }
@@ -1061,6 +1048,13 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::CrossRel& crossR
     case ::substrait::CrossRel_JoinType_JOIN_TYPE_INNER:
     case ::substrait::CrossRel_JoinType_JOIN_TYPE_LEFT:
       break;
+    case ::substrait::CrossRel_JoinType_JOIN_TYPE_OUTER:
+      if (crossRel.has_expression()) {
+        LOG_VALIDATION_MSG("Full outer join type with condition is not supported in CrossRel");
+        return false;
+      } else {
+        break;
+      }
     default:
       LOG_VALIDATION_MSG("Unsupported Join type in CrossRel");
       return false;
@@ -1084,7 +1078,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::CrossRel& crossR
 
   if (crossRel.has_expression()) {
     auto expression = exprConverter_->toVeloxExpr(crossRel.expression(), rowType);
-    exec::ExprSet exprSet({std::move(expression)}, execCtx_);
+    exec::ExprSet exprSet({std::move(expression)}, execCtx_.get());
   }
 
   return true;
@@ -1310,7 +1304,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::ReadRel& readRel
     expressions.emplace_back(exprConverter_->toVeloxExpr(readRel.filter(), rowType));
     // Try to compile the expressions. If there is any unregistered function
     // or mismatched type, exception will be thrown.
-    exec::ExprSet exprSet(std::move(expressions), execCtx_);
+    exec::ExprSet exprSet(std::move(expressions), execCtx_.get());
   }
 
   return true;
@@ -1319,46 +1313,58 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::ReadRel& readRel
 bool SubstraitToVeloxPlanValidator::validate(const ::substrait::Rel& rel) {
   if (rel.has_aggregate()) {
     return validate(rel.aggregate());
-  } else if (rel.has_project()) {
-    return validate(rel.project());
-  } else if (rel.has_filter()) {
-    return validate(rel.filter());
-  } else if (rel.has_join()) {
-    return validate(rel.join());
-  } else if (rel.has_cross()) {
-    return validate(rel.cross());
-  } else if (rel.has_read()) {
-    return validate(rel.read());
-  } else if (rel.has_sort()) {
-    return validate(rel.sort());
-  } else if (rel.has_expand()) {
-    return validate(rel.expand());
-  } else if (rel.has_generate()) {
-    return validate(rel.generate());
-  } else if (rel.has_fetch()) {
-    return validate(rel.fetch());
-  } else if (rel.has_top_n()) {
-    return validate(rel.top_n());
-  } else if (rel.has_window()) {
-    return validate(rel.window());
-  } else if (rel.has_write()) {
-    return validate(rel.write());
-  } else if (rel.has_windowgrouplimit()) {
-    return validate(rel.windowgrouplimit());
-  } else if (rel.has_set()) {
-    return validate(rel.set());
-  } else {
-    LOG_VALIDATION_MSG("Unsupported relation type: " + rel.GetTypeName());
-    return false;
   }
+  if (rel.has_project()) {
+    return validate(rel.project());
+  }
+  if (rel.has_filter()) {
+    return validate(rel.filter());
+  }
+  if (rel.has_join()) {
+    return validate(rel.join());
+  }
+  if (rel.has_cross()) {
+    return validate(rel.cross());
+  }
+  if (rel.has_read()) {
+    return validate(rel.read());
+  }
+  if (rel.has_sort()) {
+    return validate(rel.sort());
+  }
+  if (rel.has_expand()) {
+    return validate(rel.expand());
+  }
+  if (rel.has_generate()) {
+    return validate(rel.generate());
+  }
+  if (rel.has_fetch()) {
+    return validate(rel.fetch());
+  }
+  if (rel.has_top_n()) {
+    return validate(rel.top_n());
+  }
+  if (rel.has_window()) {
+    return validate(rel.window());
+  }
+  if (rel.has_write()) {
+    return validate(rel.write());
+  }
+  if (rel.has_windowgrouplimit()) {
+    return validate(rel.windowgrouplimit());
+  }
+  if (rel.has_set()) {
+    return validate(rel.set());
+  }
+  LOG_VALIDATION_MSG("Unsupported relation type: " + rel.GetTypeName());
+  return false;
 }
 
 bool SubstraitToVeloxPlanValidator::validate(const ::substrait::RelRoot& relRoot) {
   if (relRoot.has_input()) {
     return validate(relRoot.input());
-  } else {
-    return false;
   }
+  return false;
 }
 
 bool SubstraitToVeloxPlanValidator::validate(const ::substrait::Plan& plan) {
@@ -1370,7 +1376,8 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::Plan& plan) {
     for (const auto& rel : plan.relations()) {
       if (rel.has_root()) {
         return validate(rel.root());
-      } else if (rel.has_rel()) {
+      }
+      if (rel.has_rel()) {
         return validate(rel.rel());
       }
     }
