@@ -23,6 +23,7 @@ import org.apache.gluten.config.GlutenConfig._
 import org.apache.gluten.events.GlutenBuildInfoEvent
 import org.apache.gluten.exception.GlutenException
 import org.apache.gluten.extension.GlutenSessionExtensions
+import org.apache.gluten.initializer.CodedInputStreamClassInitializer
 import org.apache.gluten.task.TaskListener
 
 import org.apache.spark.{SparkConf, SparkContext, TaskFailedReason}
@@ -60,7 +61,11 @@ private[gluten] class GlutenDriverPlugin extends DriverPlugin with Logging {
 
     // Register Gluten listeners
     GlutenSQLAppStatusListener.register(sc)
-    if (conf.getBoolean(GLUTEN_SOFT_AFFINITY_ENABLED, GLUTEN_SOFT_AFFINITY_ENABLED_DEFAULT_VALUE)) {
+    if (
+      conf.getBoolean(
+        GLUTEN_SOFT_AFFINITY_ENABLED.key,
+        GLUTEN_SOFT_AFFINITY_ENABLED.defaultValue.get)
+    ) {
       SoftAffinityListener.register(sc)
     }
 
@@ -75,11 +80,13 @@ private[gluten] class GlutenDriverPlugin extends DriverPlugin with Logging {
   }
 
   override def registerMetrics(appId: String, pluginContext: PluginContext): Unit = {
-    if (pluginContext.conf().getBoolean(GLUTEN_UI_ENABLED, true)) {
+    if (
+      pluginContext.conf().getBoolean(GLUTEN_UI_ENABLED.key, GLUTEN_UI_ENABLED.defaultValue.get)
+    ) {
       _sc.foreach {
         sc =>
           GlutenEventUtils.attachUI(sc)
-          logInfo("Gluten SQL Tab has attached.")
+          logInfo("Gluten SQL Tab has been attached.")
       }
     }
   }
@@ -124,8 +131,10 @@ private[gluten] class GlutenDriverPlugin extends DriverPlugin with Logging {
         "\n=============================================================="
       )
     logInfo(loggingInfo)
-    val event = GlutenBuildInfoEvent(glutenBuildInfo.toMap)
-    GlutenEventUtils.post(sc, event)
+    if (sc.getConf.getBoolean(GLUTEN_UI_ENABLED.key, GLUTEN_UI_ENABLED.defaultValue.get)) {
+      val event = GlutenBuildInfoEvent(glutenBuildInfo.toMap)
+      GlutenEventUtils.post(sc, event)
+    }
   }
 
   private def setPredefinedConfigs(conf: SparkConf): Unit = {
@@ -140,8 +149,8 @@ private[gluten] class GlutenDriverPlugin extends DriverPlugin with Logging {
 
     // adaptive custom cost evaluator class
     val enableGlutenCostEvaluator = conf.getBoolean(
-      GlutenConfig.GLUTEN_COST_EVALUATOR_ENABLED,
-      GLUTEN_COST_EVALUATOR_ENABLED_DEFAULT_VALUE)
+      GlutenConfig.COST_EVALUATOR_ENABLED.key,
+      GlutenConfig.COST_EVALUATOR_ENABLED.defaultValue.get)
     if (enableGlutenCostEvaluator) {
       val costEvaluator = "org.apache.spark.sql.execution.adaptive.GlutenCostEvaluator"
       conf.set(SQLConf.ADAPTIVE_CUSTOM_COST_EVALUATOR_CLASS.key, costEvaluator)
@@ -150,7 +159,9 @@ private[gluten] class GlutenDriverPlugin extends DriverPlugin with Logging {
     // check memory off-heap enabled and size
     val minOffHeapSize = "1MB"
     if (
-      !conf.getBoolean(GlutenConfig.GLUTEN_DYNAMIC_OFFHEAP_SIZING_ENABLED, false) &&
+      !conf.getBoolean(
+        DYNAMIC_OFFHEAP_SIZING_ENABLED.key,
+        DYNAMIC_OFFHEAP_SIZING_ENABLED.defaultValue.get) &&
       (!conf.getBoolean(GlutenConfig.SPARK_OFFHEAP_ENABLED, false) ||
         conf.getSizeAsBytes(GlutenConfig.SPARK_OFFHEAP_SIZE_KEY, 0) < JavaUtils.byteStringAsBytes(
           minOffHeapSize))
@@ -162,11 +173,11 @@ private[gluten] class GlutenDriverPlugin extends DriverPlugin with Logging {
 
     // Session's local time zone must be set. If not explicitly set by user, its default
     // value (detected for the platform) is used, consistent with spark.
-    conf.set(GLUTEN_DEFAULT_SESSION_TIMEZONE_KEY, SQLConf.SESSION_LOCAL_TIMEZONE.defaultValueString)
+    conf.set(GLUTEN_DEFAULT_SESSION_TIMEZONE.key, SQLConf.SESSION_LOCAL_TIMEZONE.defaultValueString)
 
     // Task slots.
     val taskSlots = SparkResourceUtil.getTaskSlots(conf)
-    conf.set(GLUTEN_NUM_TASK_SLOTS_PER_EXECUTOR_KEY, taskSlots.toString)
+    conf.set(NUM_TASK_SLOTS_PER_EXECUTOR.key, taskSlots.toString)
 
     val onHeapSize: Long = conf.getSizeAsBytes(SPARK_ONHEAP_SIZE_KEY, 1024 * 1024 * 1024)
 
@@ -174,51 +185,62 @@ private[gluten] class GlutenDriverPlugin extends DriverPlugin with Logging {
     // size. Otherwise, the off-heap size is set to the value specified by the user (if any).
     // Note that this means that we will IGNORE the off-heap size specified by the user if the
     // dynamic off-heap feature is enabled.
-    val offHeapSize: Long = if (conf.getBoolean(GLUTEN_DYNAMIC_OFFHEAP_SIZING_ENABLED, false)) {
-      // Since when dynamic off-heap sizing is enabled, we commingle on-heap
-      // and off-heap memory, we set the off-heap size to the usable on-heap size. We will
-      // size it with a memory fraction, which can be aggressively set, but the default
-      // is using the same way that Spark sizes on-heap memory:
-      //
-      // spark.gluten.memory.dynamic.offHeap.sizing.memory.fraction *
-      //    (spark.executor.memory - 300MB).
-      //
-      // We will be careful to use the same configuration settings as Spark to ensure
-      // that we are sizing the off-heap memory in the same way as Spark sizes on-heap memory.
-      // The 300MB value, unfortunately, is hard-coded in Spark code.
-      ((onHeapSize - (300 * 1024 * 1024)) *
-        conf.getDouble(GLUTEN_DYNAMIC_OFFHEAP_SIZING_MEMORY_FRACTION, 0.6d)).toLong
-    } else {
-      // Optimistic off-heap sizes, assuming all storage memory can be borrowed into execution
-      // memory pool, regardless of Spark option spark.memory.storageFraction.
-      conf.getSizeAsBytes(SPARK_OFFHEAP_SIZE_KEY, 0L)
-    }
+    val offHeapSize: Long =
+      if (
+        conf.getBoolean(
+          DYNAMIC_OFFHEAP_SIZING_ENABLED.key,
+          DYNAMIC_OFFHEAP_SIZING_ENABLED.defaultValue.get)
+      ) {
+        // Since when dynamic off-heap sizing is enabled, we commingle on-heap
+        // and off-heap memory, we set the off-heap size to the usable on-heap size. We will
+        // size it with a memory fraction, which can be aggressively set, but the default
+        // is using the same way that Spark sizes on-heap memory:
+        //
+        // spark.gluten.memory.dynamic.offHeap.sizing.memory.fraction *
+        //    (spark.executor.memory - 300MB).
+        //
+        // We will be careful to use the same configuration settings as Spark to ensure
+        // that we are sizing the off-heap memory in the same way as Spark sizes on-heap memory.
+        // The 300MB value, unfortunately, is hard-coded in Spark code.
+        ((onHeapSize - (300 * 1024 * 1024)) *
+          conf.getDouble(DYNAMIC_OFFHEAP_SIZING_MEMORY_FRACTION.key, 0.6d)).toLong
+      } else {
+        // Optimistic off-heap sizes, assuming all storage memory can be borrowed into execution
+        // memory pool, regardless of Spark option spark.memory.storageFraction.
+        conf.getSizeAsBytes(SPARK_OFFHEAP_SIZE_KEY, 0L)
+      }
 
-    conf.set(GLUTEN_OFFHEAP_SIZE_IN_BYTES_KEY, offHeapSize.toString)
+    conf.set(COLUMNAR_OFFHEAP_SIZE_IN_BYTES.key, offHeapSize.toString)
     conf.set(SPARK_OFFHEAP_SIZE_KEY, offHeapSize.toString)
 
     val offHeapPerTask = offHeapSize / taskSlots
-    conf.set(GLUTEN_TASK_OFFHEAP_SIZE_IN_BYTES_KEY, offHeapPerTask.toString)
+    conf.set(COLUMNAR_TASK_OFFHEAP_SIZE_IN_BYTES.key, offHeapPerTask.toString)
 
     // If we are using dynamic off-heap sizing, we should also enable off-heap memory
     // officially.
-    if (conf.getBoolean(GLUTEN_DYNAMIC_OFFHEAP_SIZING_ENABLED, false)) {
+    if (
+      conf.getBoolean(
+        DYNAMIC_OFFHEAP_SIZING_ENABLED.key,
+        DYNAMIC_OFFHEAP_SIZING_ENABLED.defaultValue.get)
+    ) {
       conf.set(SPARK_OFFHEAP_ENABLED, "true")
 
       // We already sized the off-heap per task in a conservative manner, so we can just
       // use it.
-      conf.set(GLUTEN_CONSERVATIVE_TASK_OFFHEAP_SIZE_IN_BYTES_KEY, offHeapPerTask.toString)
+      conf.set(COLUMNAR_CONSERVATIVE_TASK_OFFHEAP_SIZE_IN_BYTES.key, offHeapPerTask.toString)
     } else {
       // Let's make sure this is set to false explicitly if it is not on as it
       // is looked up when throwing OOF exceptions.
-      conf.set(GLUTEN_DYNAMIC_OFFHEAP_SIZING_ENABLED, "false")
+      conf.set(
+        DYNAMIC_OFFHEAP_SIZING_ENABLED.key,
+        DYNAMIC_OFFHEAP_SIZING_ENABLED.defaultValueString)
 
       // Pessimistic off-heap sizes, with the assumption that all non-borrowable storage memory
       // determined by spark.memory.storageFraction was used.
       val fraction = 1.0d - conf.getDouble("spark.memory.storageFraction", 0.5d)
       val conservativeOffHeapPerTask = (offHeapSize * fraction).toLong / taskSlots
       conf.set(
-        GLUTEN_CONSERVATIVE_TASK_OFFHEAP_SIZE_IN_BYTES_KEY,
+        COLUMNAR_CONSERVATIVE_TASK_OFFHEAP_SIZE_IN_BYTES.key,
         conservativeOffHeapPerTask.toString)
     }
 
@@ -240,26 +262,6 @@ private[gluten] class GlutenDriverPlugin extends DriverPlugin with Logging {
       conf.set(SQLConf.ORC_VECTORIZED_READER_ENABLED.key, "false")
       conf.set(SQLConf.CACHE_VECTORIZED_READER_ENABLED.key, "false")
     }
-    // When the Velox cache is enabled, the Velox file handle cache should also be enabled.
-    // Otherwise, a 'reference id not found' error may occur.
-    if (
-      conf.getBoolean(COLUMNAR_VELOX_CACHE_ENABLED.key, false) &&
-      !conf.getBoolean(COLUMNAR_VELOX_FILE_HANDLE_CACHE_ENABLED.key, false)
-    ) {
-      throw new IllegalArgumentException(
-        s"${COLUMNAR_VELOX_CACHE_ENABLED.key} and " +
-          s"${COLUMNAR_VELOX_FILE_HANDLE_CACHE_ENABLED.key} should be enabled together.")
-    }
-
-    if (
-      conf.getBoolean(COLUMNAR_VELOX_CACHE_ENABLED.key, false) &&
-      conf.getSizeAsBytes(LOAD_QUANTUM.key, LOAD_QUANTUM.defaultValueString) > 8 * 1024 * 1024
-    ) {
-      throw new IllegalArgumentException(
-        s"Velox currently only support up to 8MB load quantum size " +
-          s"on SSD cache enabled by ${COLUMNAR_VELOX_CACHE_ENABLED.key}, " +
-          s"User can set ${LOAD_QUANTUM.key} <= 8MB skip this error.")
-    }
   }
 }
 
@@ -268,6 +270,7 @@ private[gluten] class GlutenExecutorPlugin extends ExecutorPlugin {
 
   /** Initialize the executor plugin. */
   override def init(ctx: PluginContext, extraConf: util.Map[String, String]): Unit = {
+    CodedInputStreamClassInitializer.modifyDefaultRecursionLimitUnsafe
     // Initialize Backend.
     Component.sorted().foreach(_.onExecutorStart(ctx))
   }
