@@ -16,10 +16,14 @@
  */
 package org.apache.gluten.extension.columnar.transition
 
+import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.exception.GlutenException
+import org.apache.gluten.extension.columnar.cost.GlutenCostModel
 
 import org.apache.spark.sql.execution.SparkPlan
 
+import scala.collection.mutable
+
 /**
  * Transition is a simple function to convert a query plan to interested [[ConventionReq]].
  *
@@ -47,9 +51,7 @@ trait Transition {
 object Transition {
   val empty: Transition = (plan: SparkPlan) => plan
   private val abort: Transition = (_: SparkPlan) => throw new UnsupportedOperationException("Abort")
-  private[transition] val graph: TransitionGraph.Builder = TransitionGraph.builder()
-
-  def factory(): Factory = Factory.newBuiltin(graph.build())
+  val factory = Factory.newBuiltin()
 
   def notFound(plan: SparkPlan): GlutenException = {
     new GlutenException(s"No viable transition found from plan's child to itself: $plan")
@@ -74,16 +76,32 @@ object Transition {
       transition.isEmpty
     }
 
-    protected def findTransition(from: Convention, to: ConventionReq)(
+    def update(body: TransitionGraph.Builder => Unit): Unit
+
+    protected[Factory] def findTransition(from: Convention, to: ConventionReq)(
         orElse: => Transition): Transition
   }
 
   private object Factory {
-    def newBuiltin(graph: TransitionGraph): Factory = {
-      new BuiltinFactory(graph)
+    def newBuiltin(): Factory = {
+      new BuiltinFactory()
     }
 
-    private class BuiltinFactory(graph: TransitionGraph) extends Factory {
+    private class BuiltinFactory() extends Factory {
+      private val graphBuilder: TransitionGraph.Builder = TransitionGraph.builder()
+      // Use of this cache allows user to set a new cost model in the same Spark session,
+      // then the new cost model will take effect for new transition-finding requests.
+      private val graphCache = mutable.Map[String, TransitionGraph]()
+
+      private def graph(): TransitionGraph = synchronized {
+        val aliasOrClass = GlutenConfig.get.rasCostModel
+        graphCache.getOrElseUpdate(
+          aliasOrClass, {
+            val base = GlutenCostModel.find(aliasOrClass)
+            graphBuilder.build(TransitionGraph.asTransitionCostModel(base))
+          })
+      }
+
       override def findTransition(from: Convention, to: ConventionReq)(
           orElse: => Transition): Transition = {
         assert(
@@ -104,7 +122,7 @@ object Transition {
               case Convention.RowType.None =>
                 // Input query plan doesn't have recognizable row-based output,
                 // find columnar-to-row transition.
-                graph.transitionOfOption(from.batchType, toRowType).getOrElse(orElse)
+                graph().transitionOfOption(from.batchType, toRowType).getOrElse(orElse)
               case fromRowType if toRowType == fromRowType =>
                 // We have only one single built-in row type.
                 Transition.empty
@@ -117,12 +135,12 @@ object Transition {
               case Convention.BatchType.None =>
                 // Input query plan doesn't have recognizable columnar output,
                 // find row-to-columnar transition.
-                graph.transitionOfOption(from.rowType, toBatchType).getOrElse(orElse)
+                graph().transitionOfOption(from.rowType, toBatchType).getOrElse(orElse)
               case fromBatchType if toBatchType == fromBatchType =>
                 Transition.empty
               case fromBatchType =>
                 // Find columnar-to-columnar transition.
-                graph.transitionOfOption(fromBatchType, toBatchType).getOrElse(orElse)
+                graph().transitionOfOption(fromBatchType, toBatchType).getOrElse(orElse)
             }
           case (ConventionReq.RowType.Any, ConventionReq.BatchType.Any) =>
             Transition.empty
@@ -132,6 +150,11 @@ object Transition {
         }
         out
       }
+
+      override def update(func: TransitionGraph.Builder => Unit): Unit = synchronized {
+        func(graphBuilder)
+        graphCache.clear()
+      }
     }
   }
 }
