@@ -52,7 +52,7 @@ case class InsertTransitions(convReq: ConventionReq) extends Rule[SparkPlan] {
           child
         } else {
           val transition =
-            Transition.factory().findTransition(from, convReq, Transition.notFound(node))
+            Transition.factory.findTransition(from, convReq, Transition.notFound(node))
           val newChild = transition.apply(child)
           newChild
         }
@@ -100,8 +100,7 @@ object Transitions {
   def enforceReq(plan: SparkPlan, req: ConventionReq): SparkPlan = {
     val convFunc = ConventionFunc.create()
     val removed = RemoveTransitions.removeForNode(plan)
-    val transition = Transition
-      .factory()
+    val transition = Transition.factory
       .findTransition(convFunc.conventionOf(removed), req, Transition.notFound(removed, req))
     val out = transition.apply(removed)
     out
