@@ -114,19 +114,9 @@ class GlutenDateFunctionsSuite extends DateFunctionsSuite with GlutenSQLTestsTra
             df1.selectExpr(s"unix_timestamp(x, 'yyyy-MM-dd mm:HH:ss')"),
             Seq(Row(secs(ts4.getTime)), Row(null), Row(secs(ts3.getTime)), Row(null)))
 
-          // legacyParserPolicy is not respected by Gluten.
           // invalid format
-          // val invalid = df1.selectExpr(s"unix_timestamp(x, 'yyyy-MM-dd aa:HH:ss')")
-          // if (legacyParserPolicy == "legacy") {
-          //   checkAnswer(invalid,
-          //     Seq(Row(null), Row(null), Row(null), Row(null)))
-          // } else {
-          //   val e = intercept[SparkUpgradeException](invalid.collect())
-          //   assert(e.getCause.isInstanceOf[IllegalArgumentException])
-          //   assert(e.getMessage.contains(
-          //     "You may get a different result due to the upgrading to Spark"))
-          // }
-
+          val invalid = df1.selectExpr(s"unix_timestamp(x, 'yyyy-MM-dd aa:HH:ss')")
+          checkAnswer(invalid, Seq(Row(null), Row(null), Row(null), Row(null)))
           // February
           val y1 = "2016-02-29"
           val y2 = "2017-02-29"
@@ -198,53 +188,8 @@ class GlutenDateFunctionsSuite extends DateFunctionsSuite with GlutenSQLTestsTra
             df2.select(unix_timestamp(col("y"), "yyyy-MM-dd")),
             Seq(Row(secs(ts5.getTime)), Row(null)))
 
-          // Not consistent behavior with gluten + velox.
-          // invalid format
-          //        val invalid = df1.selectExpr(s"to_unix_timestamp(x, 'yyyy-MM-dd bb:HH:ss')")
-          //        val e = intercept[IllegalArgumentException](invalid.collect())
-          //        assert(e.getMessage.contains('b'))
-        }
-    }
-  }
-
-  // Ported from spark with a test case for legacy mode removed.
-  testGluten("to_timestamp") {
-    Seq("legacy", "corrected").foreach {
-      legacyParserPolicy =>
-        withSQLConf(SQLConf.LEGACY_TIME_PARSER_POLICY.key -> legacyParserPolicy) {
-          val date1 = Date.valueOf("2015-07-24")
-          val date2 = Date.valueOf("2015-07-25")
-          val ts_date1 = Timestamp.valueOf("2015-07-24 00:00:00")
-          val ts_date2 = Timestamp.valueOf("2015-07-25 00:00:00")
-          val ts1 = Timestamp.valueOf("2015-07-24 10:00:00")
-          val ts2 = Timestamp.valueOf("2015-07-25 02:02:02")
-          val s1 = "2015/07/24 10:00:00.5"
-          val s2 = "2015/07/25 02:02:02.6"
-          val ts1m = Timestamp.valueOf("2015-07-24 10:00:00.5")
-          val ts2m = Timestamp.valueOf("2015-07-25 02:02:02.6")
-          val ss1 = "2015-07-24 10:00:00"
-          val ss2 = "2015-07-25 02:02:02"
-          val fmt = "yyyy/MM/dd HH:mm:ss.S"
-          val df = Seq((date1, ts1, s1, ss1), (date2, ts2, s2, ss2)).toDF("d", "ts", "s", "ss")
-
-          checkAnswer(
-            df.select(to_timestamp(col("ss"))),
-            df.select(timestamp_seconds(unix_timestamp(col("ss")))))
-          checkAnswer(df.select(to_timestamp(col("ss"))), Seq(Row(ts1), Row(ts2)))
-          if (legacyParserPolicy == "legacy") {
-            // In Spark 2.4 and earlier, to_timestamp() parses in seconds precision and cuts off
-            // the fractional part of seconds. The behavior was changed by SPARK-27438.
-            // Ignore this test case. Velox returns null for such case.
-            // val legacyFmt = "yyyy/MM/dd HH:mm:ss"
-            // checkAnswer(df.select(to_timestamp(col("s"), legacyFmt)), Seq(
-            //  Row(ts1), Row(ts2)))
-          } else {
-            checkAnswer(df.select(to_timestamp(col("s"), fmt)), Seq(Row(ts1m), Row(ts2m)))
-          }
-          checkAnswer(df.select(to_timestamp(col("ts"), fmt)), Seq(Row(ts1), Row(ts2)))
-          checkAnswer(
-            df.select(to_timestamp(col("d"), "yyyy-MM-dd")),
-            Seq(Row(ts_date1), Row(ts_date2)))
+          val invalid = df1.selectExpr(s"to_unix_timestamp(x, 'yyyy-MM-dd bb:HH:ss')")
+          checkAnswer(invalid, Seq(Row(null), Row(null), Row(null), Row(null)))
         }
     }
   }
