@@ -16,9 +16,8 @@
  */
 package org.apache.spark.sql
 
-import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.exception.GlutenException
-import org.apache.gluten.utils.{BackendTestSettings, BackendTestUtils, SystemParameters}
+import org.apache.gluten.utils.{BackendTestSettings, BackendTestUtils}
 
 import org.apache.spark.{SparkConf, SparkException}
 import org.apache.spark.sql.catalyst.expressions.codegen.CodeGenerator
@@ -195,7 +194,6 @@ class GlutenSQLQueryTestSuite
         .set("spark.io.compression.codec", "LZ4")
         .set("spark.gluten.sql.columnar.backend.ch.worker.id", "1")
         .set("spark.gluten.sql.enable.native.validation", "false")
-        .set(GlutenConfig.GLUTEN_LIB_PATH, SystemParameters.getClickHouseLibPath)
         .set("spark.sql.files.openCostInBytes", "134217728")
         .set("spark.unsafe.exceptionOnMemoryLeak", "true")
     } else {
@@ -215,9 +213,6 @@ class GlutenSQLQueryTestSuite
     "explain-aqe.sql", // explain plan is different
     "explain-cbo.sql", // explain
     "explain.sql", // explain
-    "group-analytics.sql", // wait velox to fix issue 3357
-    "array.sql", // blocked by VELOX-5768
-    "higher-order-functions.sql", // blocked by VELOX-5768
     "udf/udf-window.sql", // Local window fixes are not added.
     "window.sql" // Local window fixes are not added.
   ) ++ otherIgnoreList
