@@ -16,9 +16,8 @@
  */
 package org.apache.spark.sql
 
-import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.exception.GlutenException
-import org.apache.gluten.utils.{BackendTestSettings, BackendTestUtils, SystemParameters}
+import org.apache.gluten.utils.{BackendTestSettings, BackendTestUtils}
 
 import org.apache.spark.{SparkConf, SparkException, SparkThrowable}
 import org.apache.spark.ErrorMessageFormat.MINIMAL
@@ -199,7 +198,6 @@ class GlutenSQLQueryTestSuite
         .set("spark.io.compression.codec", "LZ4")
         .set("spark.gluten.sql.columnar.backend.ch.worker.id", "1")
         .set("spark.gluten.sql.enable.native.validation", "false")
-        .set(GlutenConfig.GLUTEN_LIB_PATH, SystemParameters.getClickHouseLibPath)
         .set("spark.sql.files.openCostInBytes", "134217728")
         .set("spark.unsafe.exceptionOnMemoryLeak", "true")
     } else {
@@ -215,6 +213,7 @@ class GlutenSQLQueryTestSuite
 
   // 3.4 inadvertently enabled with "group-by.sql" and "group-by-ordinal.sql"
   private val udafIgnoreList = Set(
+    "udaf/udaf-group-analytics.sql",
     "udaf/udaf-group-by-ordinal.sql",
     "udaf/udaf-group-by.sql"
   )
@@ -225,9 +224,6 @@ class GlutenSQLQueryTestSuite
     "explain-aqe.sql", // explain plan is different
     "explain-cbo.sql", // explain
     "explain.sql", // explain
-    "group-analytics.sql", // wait velox to fix issue 3357
-    "array.sql", // blocked by VELOX-5768
-    "higher-order-functions.sql", // blocked by VELOX-5768
     "udf/udf-window.sql", // Local window fixes are not added.
     "window.sql" // Local window fixes are not added.
   ) ++ otherIgnoreList ++ udafIgnoreList
