@@ -16,14 +16,14 @@
  */
 package org.apache.spark.sql.execution.adaptive
 
-import org.apache.gluten.GlutenConfig
+import org.apache.gluten.config.GlutenConfig
 
 import org.apache.spark.sql.execution.SparkPlan
 
 /** This [[CostEvaluator]] is to force use the new physical plan when cost is equal. */
 case class GlutenCostEvaluator() extends CostEvaluator {
   override def evaluateCost(plan: SparkPlan): Cost = {
-    if (GlutenConfig.getConf.enableGluten) {
+    if (GlutenConfig.get.enableGluten) {
       new GlutenCost(SimpleCostEvaluator, plan)
     } else {
       SimpleCostEvaluator.evaluateCost(plan)
