@@ -16,8 +16,7 @@
  */
 package org.apache.spark.sql.execution.datasources.parquet
 
-import org.apache.gluten.GlutenConfig
-import org.apache.gluten.execution.datasource.GlutenParquetWriterInjects
+import org.apache.gluten.execution.datasource.GlutenFormatFactory
 
 import org.apache.spark.TaskContext
 import org.apache.spark.internal.Logging
@@ -75,15 +74,13 @@ class ParquetFileFormat extends FileFormat with DataSourceRegister with Logging
       job: Job,
       options: Map[String, String],
       dataSchema: StructType): OutputWriterFactory = {
-    if ("true" == sparkSession.sparkContext.getLocalProperty("isNativeApplicable")) {
-
-      // pass compression to job conf so that the file extension can be aware of it.
+    if (sparkSession.sparkContext.getLocalProperty("isNativeApplicable") == "true") {
+      // Pass compression to job conf so that the file extension can be aware of it.
       val conf = ContextUtil.getConfiguration(job)
       val parquetOptions = new ParquetOptions(options, sparkSession.sessionState.conf)
       conf.set(ParquetOutputFormat.COMPRESSION, parquetOptions.compressionCodecClassName)
       val nativeConf =
-        GlutenParquetWriterInjects
-          .getInstance()
+        GlutenFormatFactory(shortName())
           .nativeConf(options, parquetOptions.compressionCodecClassName)
 
       new OutputWriterFactory {
@@ -95,9 +92,8 @@ class ParquetFileFormat extends FileFormat with DataSourceRegister with Logging
             path: String,
             dataSchema: StructType,
             context: TaskAttemptContext): OutputWriter = {
-          GlutenParquetWriterInjects
-            .getInstance()
-            .createOutputWriter(path, dataSchema, context, nativeConf);
+          GlutenFormatFactory(shortName())
+            .createOutputWriter(path, dataSchema, context, nativeConf)
 
         }
       }
@@ -196,12 +192,7 @@ class ParquetFileFormat extends FileFormat with DataSourceRegister with Logging
       sparkSession: SparkSession,
       parameters: Map[String, String],
       files: Seq[FileStatus]): Option[StructType] = {
-    // Why if (false)? Such code requires comments when being written.
-    if ("true" == sparkSession.sparkContext.getLocalProperty("isNativeApplicable") && false) {
-      GlutenParquetWriterInjects.getInstance().inferSchema(sparkSession, parameters, files)
-    } else { // the vanilla spark case
-      ParquetUtils.inferSchema(sparkSession, parameters, files)
-    }
+    ParquetUtils.inferSchema(sparkSession, parameters, files)
   }
 
   /** Returns whether the reader will return the rows as batch or not. */
@@ -239,7 +230,7 @@ class ParquetFileFormat extends FileFormat with DataSourceRegister with Logging
       requiredSchema: StructType,
       filters: Seq[Filter],
       options: Map[String, String],
-      hadoopConf: Configuration): (PartitionedFile) => Iterator[InternalRow] = {
+      hadoopConf: Configuration): PartitionedFile => Iterator[InternalRow] = {
     hadoopConf.set(ParquetInputFormat.READ_SUPPORT_CLASS, classOf[ParquetReadSupport].getName)
     hadoopConf.set(ParquetReadSupport.SPARK_ROW_REQUESTED_SCHEMA, requiredSchema.json)
     hadoopConf.set(ParquetWriteSupport.SPARK_ROW_SCHEMA, requiredSchema.json)
@@ -329,7 +320,7 @@ class ParquetFileFormat extends FileFormat with DataSourceRegister with Logging
       // have different writers.
       // Define isCreatedByParquetMr as function to avoid unnecessary parquet footer reads.
       def isCreatedByParquetMr: Boolean =
-        footerFileMetaData.getCreatedBy().startsWith("parquet-mr")
+        footerFileMetaData.getCreatedBy.startsWith("parquet-mr")
 
       val convertTz =
         if (timestampConversion && !isCreatedByParquetMr) {
@@ -520,7 +511,7 @@ object ParquetFileFormat extends Logging {
             // when it can't read the footer.
             Some(
               new Footer(
-                currentFile.getPath(),
+                currentFile.getPath,
                 ParquetFooterReader.readFooter(conf, currentFile, SKIP_ROW_GROUPS)))
           } catch {
             case e: RuntimeException =>
